package com.gx.obe.struct.composite;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.button.EvaluationButton;
import com.gx.obe.common.widget.text.EvalFactorText;
import com.gx.obe.struct.dialog.EditOpinionShell;
import com.gx.obe.struct.enums.EvaluationFactorTypeEnum;
import com.gx.obe.struct.listener.EditOpinionListener;
import com.gx.obe.struct.listener.ReturnListener;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.service.SupplierEvaluationFactorService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class ClearBidReviewEntryComposite extends Composite {
	
	private final SupplierEvaluationFactorService supplierEvaluationFactorService = new SupplierEvaluationFactorService();
	
	private final Prop<EvaluationFactor> evaluationFactorProp = new Prop<>();
	
	private final Prop<EvaluationContentData> evaluationContentData = new Prop<>();
	private final List<ReturnListener> returnListenerList = new ArrayList<>();
	
	private class EvaluationContentData {
		private String supplierId;
		private String factorCode;
		private EvaluationFactorTypeEnum factorTypeEnum;
	}
	private EvaluationFactor curFactor;
	private SupplierEvaluationFactor supplierEvaluationFactor;
	private Composite factorOpinionComposite;
	private Text opinionText;
	private Text supplierOpinionText;
	private ImageButton opinionBtn;

	private ImageButton supplierOpinionBtn;

	private EvaluationButton saveBtn;

	private Label wordCountLabel;

	private ELink editOpinionLink;

	private EvaluationButton backBtn;
	
	public ClearBidReviewEntryComposite(Composite parent, String tenderId) {
		super(parent, SWT.NONE);
		
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gridLayout = new GridLayout(3, false);
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		EvaluationContentComposite composite = new EvaluationContentComposite(this, tenderId);
		GridData gd_composite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite.widthHint = 600;
		composite.setLayoutData(gd_composite);
		evaluationContentData.bind(t -> composite.refresh(t.supplierId, t.factorCode, t.factorTypeEnum));
		
		ESepator label = new ESepator(this, SWT.VERTICAL);
		label.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, true, 1, 1));
		label.setBackground(ColorConstants.BORDER_LINE_COLOR);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		GridLayout gl_composite_1 = new GridLayout(1, false);
		gl_composite_1.verticalSpacing = 1;
		gl_composite_1.marginHeight = 0;
		gl_composite_1.marginWidth = 0;
		composite_1.setLayout(gl_composite_1);
		GridData gd_composite_1 = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite_1.widthHint = 400;
		composite_1.setLayoutData(gd_composite_1);
		
		Composite composite_2 = new Composite(composite_1, SWT.NONE);
		composite_2.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_2.setLayout(new GridLayout(1, false));
		GridData gd_composite_2 = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite_2.heightHint = 260;
		composite_2.setLayoutData(gd_composite_2);
		
		EvalFactorText text = new EvalFactorText(composite_2);
		text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		evaluationFactorProp.bind(text::refresh);
		
		Composite selfOpinionComposite = new Composite(composite_1, SWT.NONE);
		selfOpinionComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_selfOpinion = new GridLayout(1, false);
		gl_selfOpinion.marginWidth = 0;
		gl_selfOpinion.marginHeight = 0;
		selfOpinionComposite.setLayout(gl_selfOpinion);
		selfOpinionComposite.setBackground(SWTResourceManager.getColor(240, 240, 240));
		selfOpinionComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		
		supplierOpinionBtn = new ImageButton(selfOpinionComposite, SWT.NONE);
		supplierOpinionBtn.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		supplierOpinionBtn.setLeftMargin(20);
		supplierOpinionBtn.setRightMargin(20);
		supplierOpinionBtn.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		supplierOpinionBtn.setText("自评意见");
		setanswerBtnCss(supplierOpinionBtn);
		
		Composite selfOpinionTextComposite = new Composite(composite_1, SWT.NONE);
		selfOpinionTextComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		selfOpinionTextComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		GridData gd_selfOpinionTextComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_selfOpinionTextComposite.heightHint = 150;
		selfOpinionTextComposite.setLayoutData(gd_selfOpinionTextComposite);
		selfOpinionTextComposite.setLayout(new GridLayout(1, false));
		
		supplierOpinionText = new Text(selfOpinionTextComposite, SWT.WRAP | SWT.V_SCROLL | SWT.H_SCROLL);
		supplierOpinionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		supplierOpinionText.setEditable(false);
		
		Composite opinionComposite = new Composite(composite_1, SWT.NONE);
		opinionComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		GridLayout gl_opinion = new GridLayout(1, false);
		gl_opinion.marginWidth = 0;
		gl_opinion.marginHeight = 0;
		opinionComposite.setLayout(gl_opinion);
		opinionComposite.setBackground(SWTResourceManager.getColor(240, 240, 240));
		opinionComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		
		opinionBtn = new ImageButton(opinionComposite, SWT.NONE);
		opinionBtn.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		opinionBtn.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		opinionBtn.setText("评议意见");
		setanswerBtnCss(opinionBtn);
		
		factorOpinionComposite = new Composite(composite_1, SWT.NONE);
		factorOpinionComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		factorOpinionComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl_factorOpinionComposite = new GridLayout(2, false);
		gl_factorOpinionComposite.marginHeight = 0;
		gl_factorOpinionComposite.marginWidth = 0;
		factorOpinionComposite.setLayout(gl_factorOpinionComposite);
		GridData gd_factorOpinionComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_factorOpinionComposite.heightHint = 180;
		factorOpinionComposite.setLayoutData(gd_factorOpinionComposite);;
		
		opinionText = new Text(factorOpinionComposite, SWT.WRAP | SWT.V_SCROLL);
		opinionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));;
		opinionText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				wordCountLabel.setText(opinionText.getText().length() +"/2000");
			}
		});
		opinionText.setTextLimit(2000);
		opinionText.setEditable(false);
		
		wordCountLabel = new Label(factorOpinionComposite, SWT.NONE);
		wordCountLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));;
		wordCountLabel.setText("0/2000");
		
		editOpinionLink = new ELink(factorOpinionComposite, SWT.NONE);
		editOpinionLink.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));;
		editOpinionLink.setText("修改");
		editOpinionLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/edit.png"));
		editOpinionLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				editOpinionAction();
			}
		});
		
		Composite buttonComposite = new Composite(composite_1, SWT.NONE);
		buttonComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		buttonComposite.setLayout(new GridLayout(2, false));
		buttonComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		saveBtn = new EvaluationButton(buttonComposite, SWT.NONE);
		saveBtn.addMouseListener(MouseListener.mouseUpAdapter($ -> saveAction()));
		saveBtn.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		saveBtn.setText("保存");
		
		backBtn = new EvaluationButton(buttonComposite, SWT.NONE);
		backBtn.addMouseListener(MouseListener.mouseUpAdapter($ -> returnAction()));
		backBtn.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
		backBtn.setText("返回");
	}
	
	public void refresh(EvaluationFactor evaluationFactor, SupplierEvaluationFactor supplierEvaluationFactor, EvaluationFactorTypeEnum factorTypeEnum) {
		this.supplierEvaluationFactor = supplierEvaluationFactor;
		this.evaluationFactorProp.set(evaluationFactor);
		this.curFactor = evaluationFactor;
		EvaluationContentData data = new EvaluationContentData();
		data.supplierId = supplierEvaluationFactor.getSupplierId();
		data.factorCode = supplierEvaluationFactor.getFactorCode();
		data.factorTypeEnum = factorTypeEnum;
		this.evaluationContentData.set(data);
		saveBtn.setEnabled(false);
		opinionText.setText(ObjectUtils.getObjString(supplierEvaluationFactor.getOpinion()));
		opinionText.setToolTipText(supplierEvaluationFactor.getModifyReason());
		supplierOpinionText.setText(ObjectUtils.getObjString(supplierEvaluationFactor.getFactorResponse()));
	}
	
	  /**
     * @Description: 设置指标分类的按钮
     * @author wangxiang
     * @param imageBtn
     */
    private void setanswerBtnCss(ImageButton imageBtn) {
        imageBtn.setRightMargin(10);
        imageBtn.setLeftMargin(10);
        imageBtn.setTopMargin(5);
        imageBtn.setBottomMargin(5);
        imageBtn.setRightMargin(20);
        imageBtn.setLeftMargin(20);
        imageBtn.setTopMargin(5);
        imageBtn.setBottomMargin(5);
        imageBtn.setRightMargin(10);
        imageBtn.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
        imageBtn.setForeground(ColorConstants.SKIN_SELECTOIN_FOREGROUND);
        imageBtn.setBackgroundColor(ColorConstants.SKIN_BG);

    }
    
	public void addReturnListener(ReturnListener returnListener) {
		this.returnListenerList.add(returnListener);
	}
	
	  /**
     * 修改评审意见
     */
    private void editOpinionAction() {
    	
    	if (null != supplierEvaluationFactor) {
    		opinionText.setText(ObjectUtils.getObjString(supplierEvaluationFactor.getOpinion()));
    		final boolean saveBtnIsEnabled = saveBtn.getEnabled();
    		saveBtn.setEnabled(false);
    		EditOpinionShell editOpinionShell = new EditOpinionShell(getShell(), curFactor, supplierEvaluationFactor, new EditOpinionListener() {
    			@Override
    			public void editOpinion(String opinion, String modifyReason) {
    				saveBtn.setEnabled(true);
    				opinionText.setText(ObjectUtils.getObjString(opinion));
    				opinionText.setToolTipText(modifyReason);
    				supplierEvaluationFactor.setModifyReason(modifyReason);
    			}
				@Override
				public void cancle() {
					saveBtn.setEnabled(saveBtnIsEnabled);
				}
    		});
    		if (null != editOpinionShell && !editOpinionShell.isDisposed()) {
    			// 设置界面位置
    			editOpinionShell.setLocation(factorOpinionComposite.toDisplay(0, 0));
    			Rectangle rectangle = factorOpinionComposite.getBounds();
    			// 设置界面大小
    			editOpinionShell.setSize(rectangle.width, rectangle.height);
    			editOpinionShell.open();
    		}
    	}
    }
	
	private void saveAction() {
		if(!saveBtn.getEnabled())return;
		if(opinionText.getText().trim().length() == 0) {
			MessageDialog.openShortMessage("评议意见不能为空！");
			return;
		}
		saveBtn.setEnabled(false);
		backBtn.setEnabled(false);
		supplierEvaluationFactor.setOpinion(opinionText.getText());
		boolean saveFlag = false;
		if (StringUtils.isEmpty(supplierEvaluationFactor.getId())) {
            supplierEvaluationFactor.setId(UuidUtils.getUUID());
            saveFlag = supplierEvaluationFactorService.insert(supplierEvaluationFactor);
        } else {
        	// TODO 这里需要保存修改原因
        	saveFlag = supplierEvaluationFactorService.updateAssignProperty(supplierEvaluationFactor, new String[] { "opinion", "factorResponse","modifyReason" ,"paramValues"});
//        	saveFlag = supplierEvaluationFactorService.updateAssignProperty(supplierEvaluationFactor, new String[] { "opinion", "factorResponse" ,"paramValues"});
        }
		if(saveFlag) {
			MessageDialog.openShortMessage("保存成功！");
			
		}else {
			MessageDialog.openWarning(getShell(), "保存失败！");
			saveBtn.setEnabled(true);
		}
		backBtn.setEnabled(true);
	}
	
	private void returnAction() {
		returnListenerList.forEach(l -> l.returnAction());
	}
	
	@Override
	protected void checkSubclass() {}
}
