package com.gx.obe.struct.composite;

import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.listener.RoundBorderLineListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.call.Call;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.struct.beans.ModelData;
import com.gx.obe.struct.enums.ModelDataTypeEnum;
import com.gx.obe.struct.factory.ModelDataFactory;
import com.gx.obe.struct.handler.RelChapterTypeHandler;
import com.gx.obe.struct.inters.IModelData;
import com.gx.obe.struct.inters.IModelDataDetail;
import com.gx.obe.web.entity.evaluation.EvaluationContent;
import com.swtdesigner.SWTResourceManager;

public class ModelDataComposite extends Composite {
	
	private final Prop<String> title = new Prop<>();
	private final Prop<Integer> count = new Prop<>();
	private final Prop<ModelDataTypeEnum> modelDataTypeEnum = new Prop<>();
	private final Prop<ModelData> modelData = new Prop<>();
	
	private final Call call = new Call();
	
	public ModelDataComposite(Composite parent, String tenderId) {
		super(parent, SWT.NONE);
		
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginTop = 5;
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		GridData gd_composite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite.heightHint = 400;
		composite.setLayoutData(gd_composite);
		
		Composite composite_2 = new Composite(composite, SWT.NONE);
		GridLayout gl_composite_2 = new GridLayout(2, false);
		gl_composite_2.marginWidth = 0;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label lblNewLabel = new Label(composite_2, SWT.NONE);
		lblNewLabel.setFont(SWTResourceManager.getFont("Microsoft YaHei UI", 10, SWT.BOLD));
		title.bind(t -> {
			lblNewLabel.setText(t);
			composite_2.layout();
		});
		
		Composite composite_4 = new Composite(composite_2, SWT.NONE);
		composite_4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_composite_4 = new GridLayout(3, false);
		gl_composite_4.marginHeight = 0;
		composite_4.setLayout(gl_composite_4);
		
		Label lblNewLabel_1 = new Label(composite_4, SWT.NONE);
		lblNewLabel_1.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
		lblNewLabel_1.setText("共有");
		
		Label lblNewLabel_2 = new Label(composite_4, SWT.NONE);
		lblNewLabel_2.setForeground(SWTResourceManager.getColor(255, 93, 0));
		lblNewLabel_2.setFont(SWTResourceManager.getFont("Microsoft YaHei UI", 10, SWT.BOLD));
		count.bind(t -> {
			lblNewLabel_2.setText(String.valueOf(t));
			composite_4.layout();
		});
		
		Label lblNewLabel_3 = new Label(composite_4, SWT.NONE);
		lblNewLabel_3.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, false, 1, 1));
		title.bind(t -> {
			lblNewLabel_3.setText("条" + t + "信息");
			composite_4.layout();
		});
		
		Composite composite_3 = new Composite(composite, SWT.NONE);
		FillLayout fl_composite_3 = new FillLayout(SWT.HORIZONTAL);
		fl_composite_3.marginHeight = 1;
		fl_composite_3.marginWidth = 1;
		composite_3.setLayout(fl_composite_3);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_3.addPaintListener(new RoundBorderLineListener(ColorConstants.BORDER_LINE_COLOR));
		
		ModelDataDetailComposite composite_1 = new ModelDataDetailComposite(this, tenderId);
		composite_1.setLayout(new GridLayout(1, false));
		GridData gd_composite_1 = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_composite_1.heightHint = 500;
		composite_1.setLayoutData(gd_composite_1);
		
		IModelDataDetail modelDataDetail = composite_1.getModelDataDetail();
		call.bind(() -> modelDataDetail.refreshTitle(""));
		
		modelDataTypeEnum.bind(t -> {
			IModelData iModelData = ModelDataFactory.create(composite_3, modelDataDetail, t);
			iModelData.setCountWrite(count::set);
			modelData.bind(iModelData::refresh);
		});
	}
	
	public void refresh(EvaluationContent evaluationContent, ModelData modelData) {
		Optional.of(evaluationContent).map(EvaluationContent::getEvalPointName).ifPresent(title::set);
		Optional.of(evaluationContent).map(t -> {
			return new RelChapterTypeHandler(t.getRelChapterType()).getRelChapterType();
		}).map(ModelDataTypeEnum.MAP::get).ifPresent(modelDataTypeEnum::set);
		this.modelData.set(modelData);
		this.call.exe();
	}
	
	@Override
	protected void checkSubclass() {}
	
}
