package com.gx.obe.struct.composite;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import com.gx.obe.bind.prop.Prop;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.component.comparison.Column;
import com.gx.obe.component.comparison.ComparisonData;
import com.gx.obe.component.comparison.ComparisonTable;
import com.gx.obe.component.comparison.KTableDataSource;
import com.gx.obe.component.comparison.OtherMessageSHowTable;
import com.gx.obe.component.rx.RxSwt;
import com.gx.obe.component.utils.CompositeUtils;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.CommonEnum;
//import com.gx.obe.components.core.vo.TableXML;
//import com.gx.obe.components.core.vo.TableXML.Tr;
//import com.gx.obe.components.core.vo.TableXML.Tr.Td;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.message.Messages;
import com.gx.obe.struct.beans.BidderBasicInfo;
import com.gx.obe.struct.beans.Finance;
import com.gx.obe.struct.beans.ModelData;
import com.gx.obe.struct.beans.Performance;
import com.gx.obe.struct.beans.TableXML2;
import com.gx.obe.struct.beans.Td;
import com.gx.obe.struct.beans.TemplateTable;
import com.gx.obe.struct.beans.Tr;
import com.gx.obe.struct.enums.DataCategoryEnum;
import com.gx.obe.struct.service.BidderInfoService;
import com.gx.obe.struct.service.FinanceService;
import com.gx.obe.struct.service.ModelDataService;
import com.gx.obe.struct.service.PerformanceService;
import com.gx.obe.struct.service.TemplateTableService;
import com.gx.obe.struct.utils.Bean2MapUtils;
import com.gx.obe.struct.utils.GetTypeUtils;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.ProjectRuleService;

public class TemplateTableComparisonComposite extends Composite {
	
	private final TemplateTableService templateTableService = new TemplateTableService();
	private final PerformanceService performanceService = new PerformanceService();
	private final FinanceService financeService = new FinanceService();
	private final ModelDataService modelDataService = new ModelDataService();
	private final BidderInfoService bidderInfoService = new BidderInfoService();
	
	private final Prop<ComparisonData> comparisonData = new Prop<>();
	private KTableDataSource  kTableDataSource = null;
	private List<Map<String, String>> rowList;
	
	public TemplateTableComparisonComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new FillLayout(SWT.HORIZONTAL));
		
	}
	private void init(String dataCategory, String relChapterType) {
		if(dataCategory.equals("MD") && ( relChapterType.equals("EQ_BidderBasicInfo") || relChapterType.equals("EQ_FinancialReport") || relChapterType.contains("EQ_Performance_"))) {
			CompositeUtils.disposeChildren(this);
			new OtherMessageSHowTable(this, kTableDataSource);
			this.layout();
//			kTableDataSource.bind(t -> {
//				this.layout();
//			});
			
		}else {
			comparisonData.bind(t -> {
				CompositeUtils.disposeChildren(this);
				new ComparisonTable(this, t);
				this.layout();
			});
		}
	}
	public void refresh(String tenderId, String relChapterType, String dataCode, String dataCategory, List<Supplier> supplierList) {
		if(dataCategory.equals(DataCategoryEnum.MD.getKey())) {
			if( relChapterType.equals("EQ_BidderBasicInfo") || relChapterType.equals("EQ_FinancialReport") || relChapterType.contains("EQ_Performance_")) {
				kTableDataSource = getEQ_FinancialReportData(tenderId, relChapterType, dataCode, supplierList);
				Display.getDefault().syncExec(new Runnable() {
					public void run() {
						init(dataCategory, relChapterType);
					}
				});
			}else {
				init(dataCategory, relChapterType);
				RxSwt.run(() -> getComparisonData(tenderId, relChapterType, dataCode, supplierList)).checkWidget(this).exe(comparisonData::set);
			}
			
		}else if(dataCategory.equals(DataCategoryEnum.TT.getKey())){
			init(dataCategory, relChapterType);
			RxSwt.run(() -> getComparisonData(tenderId, relChapterType, dataCode, supplierList)).checkWidget(this).exe(comparisonData::set);
		}
	}
	
	/**
	 * 小范本类型数据加载
	 * @param tenderId
	 * @param relChapterType
	 * @param dataCode
	 * @param supplierList
	 * @return
	 */
	private ComparisonData getComparisonData(String tenderId, String relChapterType, String dataCode, List<Supplier> supplierList) {
		/**
		 * 小范本解析
		 */
		//获取解析入库的小范本数据
		List<TemplateTable> templateTableList = templateTableService.getTemplateTableList(tenderId, relChapterType, dataCode);
		if (CollectionUtils.isNull(templateTableList)) {
			return null;
		}
//		List<Tr> trList = null;
		int rowCount = 0;
		//获取供应商行数最多作为展示表格的行数
		Map<String, TemplateTable> templateTableMap = templateTableList.stream().collect(Collectors.toMap(TemplateTable::getSupplierId, Function.identity()));
		for(TemplateTable templateTable : templateTableList) {
			TableXML2 tableXML = JsonUtil.jsonToBean(templateTable.getTableInfo(), TableXML2.class);
			if(rowCount < tableXML.getBodyList().size()) {
				rowCount = tableXML.getBodyList().size();
			}
		}
		//获取招标文件的小范本，解析表头
		TemplateTable templateTable = templateTableMap.get(Constants.BIDDING_ID);
		if (templateTable == null) {
			return null;
		}
		//表头解析
		TableXML2 tableXML = JsonUtil.jsonToBean(templateTable.getTableInfo(), TableXML2.class);
		if (tableXML == null) {
			return null;
		}
		List<Tr> headList = tableXML.getHeadList();
		if (headList.size() != 1) {
			return null;
		}
		Tr head = headList.get(0);
		ComparisonData comparisonData = new ComparisonData();
//		for(Tr tr : tableXML.getBodyList()) {
//			System.out.println(getContent(tr, indexCol.getAsInt()));
//			comparisonData.addRow(getContent(tr, indexCol.getAsInt()));
//		}
		for(int index = 0; index < rowCount; index++) {
			comparisonData.addRow((index + 1) + "");
		}
		//表头列
		for(Supplier supplier : supplierList) {
			comparisonData.addTitleCol(new Column(supplier.getId(), supplier.getSupplierName()));
		}
		//供应商数据插入
		for(int index = 0;index < head.getTds().length; index++) {
			comparisonDataInit(templateTableMap, tableXML, head,  index, supplierList, comparisonData);
		}
		return comparisonData;
	}
	
	private void comparisonDataInit(Map<String, TemplateTable> templateTableMap, TableXML2 tableXML, Tr head,  int col, List<Supplier> supplierList, ComparisonData comparisonData) {
		Td td = head.getTds()[col];
		for(Supplier supplier : supplierList) {
			supplierComparisonDataInit(templateTableMap, col, supplier, comparisonData);
		}
		comparisonData.addItemCol(new Column(col + "", td.getContent()));

	}
	
	private void supplierComparisonDataInit(Map<String, TemplateTable> templateTableMap,  int col, Supplier supplier, ComparisonData comparisonData) {
		TemplateTable supplierTemplateTable = templateTableMap.get(supplier.getId());
		if (supplierTemplateTable == null) {
			return;
		}
		TableXML2 supplierTableXML = JsonUtil.jsonToBean(supplierTemplateTable.getTableInfo(), TableXML2.class);
//		System.out.println(supplierTemplateTable.getTableInfo());
		if (supplierTableXML == null) {
			return;
		}
//		
		int trIndex = 1;
		for(Tr  tr: supplierTableXML.getBodyList()) {
			int tdIndex = 0;
			for(Td td : tr.getTds()) {
				comparisonData.putComparisonContent(trIndex + "", supplier.getId(), tdIndex + "", td.getContent());
//				System.out.println(" put ：" + tdIndex + " : "+  supplier.getId() +" : "+  col + "; " +  td.getContent());
				tdIndex++;
				
			}
			trIndex++;
		}
//			
	}
	
	// 财务报告
	/**
	 * 小范本类型数据加载
	 * @param tenderId
	 * @param relChapterType
	 * @param dataCode
	 * @param supplierList
	 * @return
	 */
	private KTableDataSource getEQ_FinancialReportData(String tenderId, String relChapterType, String dataCode, List<Supplier> supplierList) {
		 ProjectRuleService projectRuleService = new ProjectRuleService();
		 TenderProjectRuleEntity tenderProjectRuleEntity = projectRuleService.getProjectRule(tenderId);
		KTableDataSource kTableDataSource = new KTableDataSource();
		List<String> titleList = new ArrayList<>();
		titleList.add(Messages.Num);
//		
		if( relChapterType.equals("EQ_BidderBasicInfo")) {
			//投标人基本信息
//			读取xml模版信息
			File templateFile = FileConstants.getResourceFile(Constants.TYPE_BIDDER);
			Map<String, List<Map<String, String>>> typeMap = GetTypeUtils.resolverPerformance(tenderId, templateFile.getAbsolutePath());
			rowList = typeMap.get(tenderProjectRuleEntity.getParamByKey("professionalCode"));
//			titleList.add(Messages.LoginDialog_label_2_text);
			for(Map<String, String> map : rowList) {
				titleList.add(map.get("name"));
			}
		}else if(relChapterType.equals("EQ_FinancialReport")) {
			//财务指标
			
			titleList.add(Messages.LoginDialog_label_15_text);
			titleList.add(Messages.LoginDialog_label_16_text);
			titleList.add(Messages.LoginDialog_label_17_text);
			titleList.add(Messages.LoginDialog_label_18_text);
			titleList.add(Messages.LoginDialog_label_19_text);
			titleList.add(Messages.LoginDialog_label_20_text);
			titleList.add(Messages.LoginDialog_label_21_text);
			
		}else if(relChapterType.contains("EQ_Performance_")) {
			
			File templateFile = FileConstants.getResourceFile(Constants.TYPE_YEJI);
			Map<String, List<Map<String, String>>> typeMap = GetTypeUtils.resolverPerformance(tenderId, templateFile.getAbsolutePath());
			rowList = typeMap.get(tenderProjectRuleEntity.getParamByKey("professionalCode"));
//			titleList.add(Messages.LoginDialog_label_2_text);
			for(Map<String, String> map : rowList) {
			
				titleList.add(map.get("name"));
			}
		}
		kTableDataSource.setTitleList(titleList);
		int rowIndex = 1;
		for(Supplier supplier : supplierList) {
			ModelData modelData  = modelDataService.getModelData(tenderId, supplier.getId(), relChapterType, dataCode);
			if(null != modelData ) {
				if( relChapterType.equals("EQ_BidderBasicInfo")) {
					List<BidderBasicInfo> bidderBaseInfoList = bidderInfoService.getBidderBasicInfoList(tenderId, modelData.getId());
					for(BidderBasicInfo bidderBasicInfo : bidderBaseInfoList) {
						List<Object> supplierMessage = new ArrayList<>();
						supplierMessage.add(rowIndex);
//						supplierMessage.add(supplier.getSupplierName());
						Map<String, String> bidderBasicInfoMap = Bean2MapUtils.createMap(bidderBasicInfo);
						for(Map<String, String> map : rowList) {
							
							supplierMessage.add(ObjectUtils.getObjString(bidderBasicInfoMap.get(map.get("code"))));
						}
						kTableDataSource.addRowList(supplierMessage);
						rowIndex ++;
					}
					
					
				}else if(relChapterType.equals("EQ_FinancialReport")) {
					List<Finance> finaceList = financeService.getFinanceList(tenderId, modelData.getId());
//					List<BidderBasicInfo> bidderBaseInfoList = bidderInfoService.getBidderBasicInfoList(tenderId, modelData.getId());
					for(Finance finance : finaceList) {
						List<Object> supplierMessage = new ArrayList<>();
						supplierMessage.add(rowIndex);
						supplierMessage.add(supplier.getSupplierName());
						supplierMessage.add(finance.getAnnual());
						supplierMessage.add(finance.getTotal());
						supplierMessage.add(finance.getTotalliabilities());
						supplierMessage.add(finance.getLiabilitiesRate());
						supplierMessage.add(finance.getPrimeOperatingRevenue());
						supplierMessage.add(finance.getRetainedProfits());
						supplierMessage.add(finance.getProfitRatio());
						kTableDataSource.addRowList(supplierMessage);
						rowIndex ++;
					}
				}else if(relChapterType.contains("EQ_Performance_")) {
					String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
					List<Performance> performanceList = performanceService.getPerformanceList(tenderId, modelData.getId(), statue);
					for(Performance performance : performanceList) {
						List<Object> supplierMessage = new ArrayList<>();
						supplierMessage.add(rowIndex);
//						supplierMessage.add(supplier.getSupplierName());
						Map<String, String> bidderBasicInfoMap = Bean2MapUtils.createMap(performance);
						for(Map<String, String> map : rowList) {
							String code = map.get("code");
							if(null != code && code.equals("status") ) {
								if(null != performance.getStatus() && CommonEnum.YES.equals(performance.getStatus())) {
									supplierMessage.add("已完成");
								}else {
									supplierMessage.add("未完成");
								}
							}else {
//								supplierMessage.add(bidderBasicInfoMap.get(map.get("code")));
								supplierMessage.add(ObjectUtils.getObjString(bidderBasicInfoMap.get(map.get("code"))));
							}
						}
						kTableDataSource.addRowList(supplierMessage);
						
						rowIndex ++;
					}
					
				}
			}
			
		}
		return kTableDataSource;
	}
	
	
	@Override
	protected void checkSubclass() {}
	

	public static void main(String[] args) {
		Map<String, String> map = new HashMap<>();
		map.put("linkManWebsite", "www.wanguo.com");
		System.out.println(map.get("linkManWebsite"));
		
	}
}
