package com.gx.obe.struct.composite;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.common.widget.button.RectangleRadioButton;
import com.gx.obe.component.handler.TabHandler;
import com.gx.obe.component.rx.RxSwt;
import com.gx.obe.message.Messages;
import com.gx.obe.struct.enums.DataCategoryEnum;
import com.gx.obe.struct.service.EvaluationContentService;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationContent;
import com.swtdesigner.SWTResourceManager;

public class TemplateTableListComposite extends Composite {
	
	private final EvaluationContentService evaluationContentService = new EvaluationContentService();
	
	private final Prop<TemplateTableData> templateTableData = new Prop<>();
	
	private class TemplateTableData {
		private String tenderId;
		private List<EvaluationContent> evaluationContentList;
		private List<Supplier> supplierList;
	}
	
	private final Prop<TemplateTableComparisonData> templateTableComparisonData = new Prop<>();
	
	private class TemplateTableComparisonData {
		private String tenderId;
		private String relChapterType;
		private String dataCode;
		private String dataCategory;
		private List<Supplier> supplierList;
	}
	
	public TemplateTableListComposite(Composite parent, int style) {
		super(parent, style);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		StackLayout stackLayout = new StackLayout();
		setLayout(stackLayout);
		Consumer<Control> topControlConsumer = t -> {
			stackLayout.topControl = t;
			this.layout();
		};
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setLayout(new GridLayout(1, false));
		
		Label label = new Label(composite_1, SWT.WRAP);
		label.setForeground(SWTResourceManager.getColor(200, 200, 200));
		label.setFont(SWTResourceManager.getFont("宋体", 20, SWT.BOLD));
		label.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true, 1, 1));
		label.setText(Messages.LoginDialog_label_30_text);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.verticalSpacing = 0;
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		
		Composite tabComposite = new Composite(composite, SWT.NONE);
		RowLayout rl_tabComposite = new RowLayout(SWT.HORIZONTAL);
		rl_tabComposite.marginBottom = 0;
		tabComposite.setLayout(rl_tabComposite);
		tabComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		TabHandler<RectangleRadioButton> handler = new TabHandler<>(tabComposite, t -> new RectangleRadioButton(t, SWT.NONE));
		
		this.templateTableData.bind(d -> {
			if (d.evaluationContentList.isEmpty()) {
				topControlConsumer.accept(composite_1);
			} else {
				topControlConsumer.accept(composite);
				handler.start().listHand(d.evaluationContentList, EvaluationContent::getEvalPointName, t -> selectAction(d.tenderId, t, d.supplierList)).end();
			}
		});
		
		ESepator label_1 = new ESepator(composite, SWT.HORIZONTAL);
		label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label_1.setBackground(ColorConstants.SKIN_BG);
		
		TemplateTableComparisonComposite comparisonComposite = new TemplateTableComparisonComposite(composite, SWT.NONE);
		comparisonComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		this.templateTableComparisonData.bind(d -> comparisonComposite.refresh(d.tenderId, d.relChapterType, d.dataCode, d.dataCategory, d.supplierList));
	}
	
	private void selectAction(String tenderId, EvaluationContent evaluationContent, List<Supplier> supplierList) {
		TemplateTableComparisonData data = new TemplateTableComparisonData();
		data.tenderId = tenderId;
		data.relChapterType = evaluationContent.getRelChapterType();
		data.dataCode = evaluationContent.getDataCode();
		data.dataCategory = evaluationContent.getDataCategory();
		data.supplierList = supplierList;
		this.templateTableComparisonData.set(data);
	}
	
	public void refresh(String tenderId, String factorCode, List<Supplier> supplierList) {
		RxSwt.run(() -> evaluationContentService.getEvaluationContentListByFactorCode(tenderId, factorCode)).checkWidget(this).other(ArrayList::new).exe(l -> {
			TemplateTableData data = new TemplateTableData();
			data.tenderId = tenderId;
			data.evaluationContentList = l.stream().filter(t -> (DataCategoryEnum.TT.getKey().equals(t.getDataCategory()) || DataCategoryEnum.MD.getKey().equals(t.getDataCategory()))).collect(Collectors.toList());
			data.supplierList = supplierList;
			this.templateTableData.set(data);
		});
	}
	
	@Override
	protected void checkSubclass() {}
}
