package com.gx.obe.struct.dialog;

import java.math.BigDecimal;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.AbsoluteModelDO;
import com.gx.obe.struct.beans.IntelligentEvalDO;
import com.gx.obe.struct.beans.LiftRateDO;
import com.gx.obe.struct.beans.ScoreOptionsDO;
import com.gx.obe.struct.beans.SelfEvalParamsDO;
import com.gx.obe.struct.listener.EditOpinionListener;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class EditOpinionShell extends Shell {
	private Listener deactivateListener;
	private EvaluationFactor curFactor;
	private SupplierEvaluationFactor supplierEvaluationFactor;
	private IntelligentEvalDO intelligentEvalDO;
	private EditOpinionListener editOpinionListener;
	private ELink editOpinionLink;
	private ELink saveEditOpinionLink;
	private String editOpinionStr = null;
	private String editOpinionValueStr = null;
	private Composite editContentComposite;
	private Text reasonText;
	private GridData gd_editContentComposite;
	private GridLayout gl_editContentComposite;


	/**
	 * Create the shell.
	 * @param display
	 */
	public EditOpinionShell(Shell shell, EvaluationFactor curFactor, SupplierEvaluationFactor supplierEvaluationFactor, EditOpinionListener editOpinionListener) {
		super(shell, SWT.NO_TRIM);
		setBackground(ColorConstants.BORDER_LINE_COLOR);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 1;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		this.curFactor = curFactor;
		this.supplierEvaluationFactor = supplierEvaluationFactor;
		this.editOpinionListener = editOpinionListener;
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				removeListener(SWT.Deactivate, deactivateListener);
			}
		});
		
		editContentComposite = new Composite(this, SWT.NONE);
		editContentComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		editContentComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		
		gd_editContentComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		editContentComposite.setLayoutData(gd_editContentComposite);
		gl_editContentComposite = new GridLayout(1, false);
		gl_editContentComposite.marginHeight = 1;
		gl_editContentComposite.marginWidth = 1;
		editContentComposite.setLayout(gl_editContentComposite);;
		
		Composite reasonTitleComposite = new Composite(this, SWT.NONE);
		reasonTitleComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		reasonTitleComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		reasonTitleComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		reasonTitleComposite.setLayout(new GridLayout(1, false));
		
		Label reasonLabel = new Label(reasonTitleComposite, SWT.NONE);
		reasonLabel.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		reasonLabel.setText("修改原因：");
		
		Composite editReasonComposite = new Composite(this, SWT.NONE);
		editReasonComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		editReasonComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		GridData gd_editReasonComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_editReasonComposite.heightHint = 40;
		editReasonComposite.setLayoutData(gd_editReasonComposite);
		GridLayout gl_editReasonComposite = new GridLayout(3, false);
		gl_editReasonComposite.horizontalSpacing = 0;
		gl_editReasonComposite.marginHeight = 0;
		gl_editReasonComposite.marginWidth = 0;
		editReasonComposite.setLayout(gl_editReasonComposite);
		
		
		reasonText = new Text(editReasonComposite, SWT.WRAP | SWT.V_SCROLL);
		reasonText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 2));
		reasonText.setTextLimit(200);
		reasonText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
			}
		});
		
		saveEditOpinionLink = new ELink(editReasonComposite, SWT.NONE);
		saveEditOpinionLink.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, false, true, 2, 1));
		saveEditOpinionLink.setText("确认修改");
		saveEditOpinionLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/save.png"));
		saveEditOpinionLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveEditOpinionAction();
			}
		});;
		
		editOpinionLink = new ELink(editReasonComposite, SWT.NONE);
		editOpinionLink.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
		editOpinionLink.setText("取消修改");
		editOpinionLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/back.png"));
		editOpinionLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				cancleAction();
			}
		});
		createContents();
		init();
	}
	
	private void init() {
		deactivateListener = new Listener(){
			@Override
			public void handleEvent(Event arg0) {
				close();
			}
		};
		
		addListener(SWT.Deactivate, deactivateListener);
		editOpinionStr = supplierEvaluationFactor.getOpinion();
		reasonText.setText(ObjectUtils.getObjString(supplierEvaluationFactor.getModifyReason()));
		// 如果是智能评审类型
		if(EvaluationFactorEnum.SCORE_TYPE_AI.equals(curFactor.getScoreType())) {
			intelligentEvalDO = JsonUtil.jsonToBean(curFactor.getIntelligentEval(), IntelligentEvalDO.class);
		}
		if(null != intelligentEvalDO) {
			if("F".equals(intelligentEvalDO.getFactorModel())) {
				addComboOpinion();
			}else  if("B".equals(intelligentEvalDO.getFactorModel())) {
				addLiftRateOpinion();
			} else if("G".equals(intelligentEvalDO.getFactorModel())) {
				addAbsoluteParamOpinion();
			}else {
				addStringOpinion();
			}
		}
		else {
			addStringOpinion();
		}
	}
	
	/**
	 * 确认修改
	 */
	private void saveEditOpinionAction() {
		removeListener(SWT.Deactivate, deactivateListener);
		if(null != intelligentEvalDO) {
			SelfEvalParamsDO selfEvalParamsDO = JsonUtil.jsonToBean(supplierEvaluationFactor.getParamValues(), SelfEvalParamsDO.class);
			if("F".equals(intelligentEvalDO.getFactorModel())) {
				if(null != supplierEvaluationFactor.getParamValues()) {
					if(null != selfEvalParamsDO) {
						selfEvalParamsDO.setSelectParamValue(editOpinionValueStr);
						supplierEvaluationFactor.setParamValues(selfEvalParamsDO.toJson());
					}
				}
			}else if("G".equals(intelligentEvalDO.getFactorModel())) {
				 Pattern p = Pattern.compile("^\\d*[1-9]\\d*$");   
				 Matcher m = p.matcher(editOpinionValueStr);   
				if(!m.matches()){
					MessageDialog.openShortMessage("请输入正确的整数绝对值！");
					addListener(SWT.Deactivate, deactivateListener);
					return ;
				}
				if(null != selfEvalParamsDO) {
					selfEvalParamsDO.setSelectParamValue(editOpinionValueStr);
					supplierEvaluationFactor.setParamValues(selfEvalParamsDO.toJson());
				}
			}else if("B".equals(intelligentEvalDO.getFactorModel())) {
				
				BigDecimal value = null;
				try {
					value = new BigDecimal(editOpinionValueStr);
					LiftRateDO liftRateDO = intelligentEvalDO.getLiftRate();
					if(null != liftRateDO) {
						editOpinionValueStr = BigDecimalUtils.round(value, ObjectUtils.getIntValue(liftRateDO.getDecimalDigits(), 6)).toString();
						editOpinionStr = editOpinionValueStr + liftRateDO.getUnit();
					}
				} catch (Exception e) {
					MessageDialog.openShortMessage("请输入正确的数值！");
					addListener(SWT.Deactivate, deactivateListener);
					return ;
				}
				if(null != selfEvalParamsDO) {
					selfEvalParamsDO.setSelectParamValue(editOpinionValueStr);
					supplierEvaluationFactor.setParamValues(selfEvalParamsDO.toJson());
				}
			}
		}
		if(reasonText.getText().trim().length() == 0) {
			MessageDialog.openShortMessage("请输入修改原因！");
			addListener(SWT.Deactivate, deactivateListener);
			return;
		}
		supplierEvaluationFactor.setModifyReason(reasonText.getText());
		editOpinionListener.editOpinion(editOpinionStr, reasonText.getText());
		this.close();
	}
	
	/**
	 * 取消修改
	 */
	private void cancleAction() {
		this.close();
	}
	
	private void addStringOpinion() {
		gl_editContentComposite.numColumns = 1;
    	gd_editContentComposite.grabExcessVerticalSpace = true;
		gd_editContentComposite.heightHint = 60;
		
		final Text opinionText = new Text(editContentComposite, SWT.WRAP | SWT.V_SCROLL);
		opinionText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		opinionText.setText(ObjectUtils.getObjString(supplierEvaluationFactor.getOpinion()));
		opinionText.setTextLimit(2000);
		editOpinionStr = opinionText.getText();
		opinionText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				editOpinionStr = opinionText.getText().trim();
			}
		});
		editContentComposite.getParent().layout();
	}

    /**
     * 设置下拉选项评议意见
     */
    private void addComboOpinion() {
    	gl_editContentComposite.numColumns = 1;
    	gd_editContentComposite.grabExcessVerticalSpace = false;
    	gd_editContentComposite.heightHint = -1;
    	 final Combo opinionCombo = new Combo(editContentComposite, SWT.READ_ONLY);
    	 opinionCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    	 opinionCombo.addSelectionListener(new SelectionAdapter() {
    		 public void widgetSelected(SelectionEvent e) {
    			 selectOpinionAction(opinionCombo);
    		 }
    	 });
    	 
    	 String selectParamValue = null;// 下拉选项的值
		   if(null != supplierEvaluationFactor.getParamValues()) {
			   SelfEvalParamsDO selfEvalParamsDO = JsonUtil.jsonToBean(supplierEvaluationFactor.getParamValues(), SelfEvalParamsDO.class);
			   if(null != selfEvalParamsDO) {
				   selectParamValue = selfEvalParamsDO.getSelectParamValue();
				   editOpinionValueStr = selectParamValue;
			   }
		   }
		   
    	 int index = 0;
    	 List<ScoreOptionsDO> scoreOptionsDOList = intelligentEvalDO.getScoreOptionsDOList(); 
    	 if(null != scoreOptionsDOList) {
    		 for(ScoreOptionsDO scoreOptionsDO : scoreOptionsDOList) {
    			 opinionCombo.add(scoreOptionsDO.getName(), index);
    			 opinionCombo.setData(index + "", scoreOptionsDO);
    			 if(null != selectParamValue) {
    				 // 如果有下拉选项值，则通过值进行匹配
    				 if(selectParamValue.equals(scoreOptionsDO.getScoreItemId())){
    					 opinionCombo.select(index);
    				 }
    			 }else {
    				 if(scoreOptionsDO.getName().equals(supplierEvaluationFactor.getOpinion())){
    					 opinionCombo.select(index);
    				 }
    			 }
    			 index++;
    		 }
    	 }
    	 
    }
    /**
     * @Description: 修改下拉框评审意见
     * @author chenxw
     */
    private void selectOpinionAction(Combo opinionCombo) {
    	if(opinionCombo.getSelectionIndex() >= 0 && null != curFactor) {
    		if(null != supplierEvaluationFactor) {
    			ScoreOptionsDO scoreOptionsDO = (ScoreOptionsDO)opinionCombo.getData("" + opinionCombo.getSelectionIndex());
    			if(null != scoreOptionsDO) {
    				editOpinionStr = scoreOptionsDO.getName();
    				editOpinionValueStr = scoreOptionsDO.getScoreItemId();
    			}
    		}
    	}
    }
    
    /**
     * 绝对值模型修改
     */
    private void addAbsoluteParamOpinion() {
    	gl_editContentComposite.numColumns = 3;
    	gd_editContentComposite.grabExcessVerticalSpace = false;
		gd_editContentComposite.heightHint = -1;
		
		String selectParamValue = null;// 下拉选项的值
		if(null != supplierEvaluationFactor.getParamValues()) {
			SelfEvalParamsDO selfEvalParamsDO = JsonUtil.jsonToBean(supplierEvaluationFactor.getParamValues(), SelfEvalParamsDO.class);
			if(null != selfEvalParamsDO) {
				selectParamValue = selfEvalParamsDO.getSelectParamValue();
				editOpinionValueStr = selectParamValue;
			}
		}
		AbsoluteModelDO absoluteModelDO = intelligentEvalDO.getAbsoluteModelDO();
		
    	final Text opinionText = new Text(editContentComposite, SWT.CENTER | SWT.BORDER);
		GridData gd_opinionText = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1);
		gd_opinionText.widthHint = 50;
		opinionText.setLayoutData(gd_opinionText);
		opinionText.setText(ObjectUtils.getObjString(selectParamValue));
		opinionText.setTextLimit(2000);
		Label unitLabel = new Label(editContentComposite, SWT.NONE);
		unitLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
		if(null != absoluteModelDO) {
			unitLabel.setText(ObjectUtils.getObjString(absoluteModelDO.getAbsoluteUnit(),"未知单位"));
		}
		
    	Label tipsLabel = new Label(editContentComposite, SWT.NONE);
    	tipsLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    	tipsLabel.setForeground(ColorConstants.NOTE_TIPS_COLOR);
    	//<bidFillMemo><![CDATA[此处只能填写整数绝对值]]></bidFillMemo>
    	tipsLabel.setText("请填写整数绝对值！");
    	
    	opinionText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent arg0) {
				editOpinionStr = opinionText.getText().trim() + unitLabel.getText();
				editOpinionValueStr = opinionText.getText().trim();
			}
		});
    	
    }
    /**
     * 升降率模型修改
     */
    private void addLiftRateOpinion() {
    	gl_editContentComposite.numColumns = 3;
    	gd_editContentComposite.grabExcessVerticalSpace = false;
    	gd_editContentComposite.heightHint = -1;
    	
    	String selectParamValue = null;// 下拉选项的值
    	if(null != supplierEvaluationFactor.getParamValues()) {
    		SelfEvalParamsDO selfEvalParamsDO = JsonUtil.jsonToBean(supplierEvaluationFactor.getParamValues(), SelfEvalParamsDO.class);
    		if(null != selfEvalParamsDO) {
    			selectParamValue = selfEvalParamsDO.getSelectParamValue();
    			editOpinionValueStr = selectParamValue;
    		}
    	}
    	
    	LiftRateDO liftRateDO = intelligentEvalDO.getLiftRate();
    	
    	final Text opinionText = new Text(editContentComposite, SWT.CENTER | SWT.BORDER);
    	GridData gd_opinionText = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1);
    	gd_opinionText.widthHint = 100;
    	opinionText.setLayoutData(gd_opinionText);
//    	<unit>%</unit>
//        <decimalDigits>6</decimalDigits>
    	opinionText.setText(ObjectUtils.getObjString(selectParamValue));
    	opinionText.setTextLimit(2000);
    	Label unitLabel = new Label(editContentComposite, SWT.NONE);
    	unitLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
    	if(null != liftRateDO) {
    		unitLabel.setText(ObjectUtils.getObjString(liftRateDO.getUnit(),"未知单位"));
    	}
    	
    	Label tipsLabel = new Label(editContentComposite, SWT.NONE);
    	tipsLabel.setForeground(ColorConstants.NOTE_TIPS_COLOR);
    	tipsLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    	//<bidFillMemo><![CDATA[此处应填写数值，小数点保留6位]]></bidFillMemo>
    	tipsLabel.setText("请填写数值（小数点保留"+liftRateDO.getDecimalDigits()+"位）！");
    	
    	opinionText.addModifyListener(new ModifyListener() {
    		public void modifyText(ModifyEvent arg0) {
    			editOpinionStr = opinionText.getText().trim() + unitLabel.getText();
    			editOpinionValueStr = opinionText.getText().trim();
    		}
    	});
    	
    }
   

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText("SWT Application");
		setSize(450, 300);

	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
