package com.gx.obe.struct.dialog;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.promise.Promise;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.bind.prop.RProp;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.component.grid.FGrid;
import com.gx.obe.component.listener.SelectionListener;
import com.gx.obe.component.rx.RxSwt;
import com.gx.obe.components.core.enumeration.ExpertEnum;
import com.gx.obe.struct.enums.EvaluationFactorTypeEnum;
import com.gx.obe.web.entity.expert.Expert;
import com.gx.obe.web.service.expert.ExpertService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class SelectStepExpertDialog extends EDialog {
	
	private final List<SelectionListener<Expert>> selectionListeners = new ArrayList<>();
	
	private final ExpertService expertService = new ExpertService();
	
	private final String tenderId;
	private final EvaluationFactorTypeEnum evaluationFactorTypeEnum;
	
	private Prop<List<Expert>> expertList = new Prop<>();
	private RProp<Expert> expert = new RProp<>();
	
	public SelectStepExpertDialog(Shell parentShell, String tenderId, EvaluationFactorTypeEnum evaluationFactorTypeEnum) {
		super(parentShell);
		this.tenderId = tenderId;
		this.evaluationFactorTypeEnum = evaluationFactorTypeEnum;
		setTitle("选择评标专家");
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setBackground(ColorConstants.BORDER_LINE_COLOR);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.horizontalSpacing = 0;
		gl_container.verticalSpacing = 1;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_1.setLayout(new GridLayout(1, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		composite_1.setBackgroundMode(SWT.INHERIT_FORCE);
		
		CLabel label_3 = new CLabel(composite_1, SWT.NONE);
		label_3.setFont(SWTResourceManager.getFont("宋体", 10, SWT.BOLD));
		label_3.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/television.png"));
		label_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label_3.setText("评标专家");
		
		Composite composite_2 = new Composite(container, SWT.NONE);
		composite_2.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_composite_2 = new GridLayout(1, false);
		gl_composite_2.marginHeight = 0;
		gl_composite_2.marginWidth = 0;
		composite_2.setLayout(gl_composite_2);
		
		FGrid<Expert> composite_3 = new FGrid<>(composite_2);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_3.addIndexColumn("序号").setWidth(50);
		composite_3.addColumn("姓名", t -> t.getUser().getUserName(), SWT.CENTER).setWidth(100);
		composite_3.addColumn("账号", t -> t.getUser().getUserAccount(), SWT.CENTER).setWidth(100);
		composite_3.addColumn("专家类型", Expert::getExpertTypeAndRole, SWT.CENTER).setWidth(250);
		expertList.bind(composite_3::refresh);
		expert.reader(composite_3);
		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite.setBackgroundMode(SWT.INHERIT_FORCE);
		composite.setLayout(new GridLayout(2, false));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		EButton button_1 = new EButton(composite, SWT.NONE);
		button_1.addMouseListener(MouseListener.mouseUpAdapter($ -> confirmAction()));
		button_1.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/yes.png"));
		button_1.setText("确定");
		button_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		
		EButton button = new EButton(composite, SWT.NONE);
		button.addMouseListener(MouseListener.mouseUpAdapter($ -> close()));
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		button.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		button.setText("关闭");
		
		init();
		
		return container;
	}
	
	private void init() {
		RxSwt.run(this::queryExpertList).checkWidget(getShell()).exe(expertList::set);
	}
	
	public List<Expert> queryExpertList() {
		List<Expert> experts = expertService.getExpertList(tenderId, ExpertEnum.ALL);
		if (experts == null) {
			return null;
		}
		return experts.stream().filter(getExpertPredicate()).collect(Collectors.toList());
	}
	
	private Predicate<Expert> getExpertPredicate() {
		switch (evaluationFactorTypeEnum) {
			case COMPLIANCE:
				return t -> true;
			case BUSINESS:
				return t -> (t.getEvalBidTypeNotNull() & ExpertEnum.EXPERT_BUS) != 0;
			case TECHNOLOGY:
				return t -> (t.getEvalBidTypeNotNull() & ExpertEnum.EXPERT_TEC) != 0;
			default:
				return t -> false;
		}
	}
	
	private void confirmAction() {
		close();
		selectionListeners.forEach(l -> l.selection(expert.get()));
	}
	
	public void addSelectionListener(SelectionListener<Expert> listener) {
		selectionListeners.add(listener);
	}
	
	public static Promise<Expert> selectExpert(Shell parentShell, String tenderId, EvaluationFactorTypeEnum evaluationFactorTypeEnum) {
		SelectStepExpertDialog dialog = new SelectStepExpertDialog(parentShell, tenderId, evaluationFactorTypeEnum);
		return new Promise<>((s, f) -> {
			dialog.addSelectionListener(s::accept);
			dialog.open();
		});
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(600, 400);
	}
}
