package com.gx.obe.struct.factory;

import java.io.File;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Stream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.component.form.FForm;
import com.gx.obe.component.rx.RxSwt;
import com.gx.obe.component.table.FTable;
import com.gx.obe.component.utils.CompositeUtils;
import com.gx.obe.components.core.Constants;
import com.gx.obe.struct.beans.AttachmentFile;
import com.gx.obe.struct.beans.BidderBasicInfo;
import com.gx.obe.struct.beans.BusinessLicense;
import com.gx.obe.struct.beans.Certificate;
import com.gx.obe.struct.beans.Finance;
import com.gx.obe.struct.beans.ModelData;
import com.gx.obe.struct.beans.Performance;
import com.gx.obe.struct.beans.ProjectLeader;
import com.gx.obe.struct.beans.Qualification;
import com.gx.obe.struct.beans.WorkExperience;
import com.gx.obe.struct.enums.ModelDataTypeEnum;
import com.gx.obe.struct.enums.PerformanceStatus;
import com.gx.obe.struct.inters.IModelData;
import com.gx.obe.struct.inters.IModelDataDetail;
import com.gx.obe.struct.service.AttachmentFileService;
import com.gx.obe.struct.service.BidderInfoService;
import com.gx.obe.struct.service.BusinessLicenseService;
import com.gx.obe.struct.service.CertificateService;
import com.gx.obe.struct.service.FinanceService;
import com.gx.obe.struct.service.PerformanceService;
import com.gx.obe.struct.service.ProjectLeaderService;
import com.gx.obe.struct.service.QualificationService;
import com.gx.obe.struct.service.WorkExperienceService;
import com.gx.obe.util.utils.NumberFormatUtils;
import com.gx.obe.util.utils.RuntimeUtils;
import com.swtdesigner.ResourceManager;

public class ModelDataFactory {
	
	public static IModelData create(Composite tableParent, IModelDataDetail modelDataDetail, ModelDataTypeEnum modelDataTypeEnum) {
		CompositeUtils.disposeChildren(tableParent);
		switch (modelDataTypeEnum) {
			case EQ_BidderBasicInfo:
				return initBidderBasicInfo(tableParent, modelDataDetail);
			case EQ_BusinessLicense:
				return initBusinessLicense(tableParent, modelDataDetail);
//			case EQ_Construction:
//				return initConstruction(tableParent, modelDataDetail);
//			case EQ_FinancialReport:
//				return initFinancialReport(tableParent, modelDataDetail);
//			case EQ_Procurement:
//				return initProcurement(tableParent, modelDataDetail);
//			case EQ_Production:
//				return initProduction(tableParent, modelDataDetail);
			case EQ_ProjectLeader:
				return initProjectLeader(tableParent, modelDataDetail);
			case EQ_Qualification:
				return initQualification(tableParent, modelDataDetail);
			case EQ_Qualification_GJ_GCJX:
				return initQualification_GJ_GCJX(tableParent, modelDataDetail);
			case EQ_Qualification_GJ_JX:
				return initQualification_GJ_JX(tableParent, modelDataDetail);
			case EQ_Qualification_HBTX:
				return initQualification_HBTX(tableParent, modelDataDetail);
			case EQ_Qualification_JLZZ:
				return initQualification_JLZZ(tableParent, modelDataDetail);
			case EQ_Qualification_QGGYSCXK:
				return initQualification_QGGYSCXK(tableParent, modelDataDetail);
			case EQ_Qualification_SCXK:
				return initQualification_SCXK(tableParent, modelDataDetail);
			case EQ_Qualification_SGZZ:
				return initQualification_SGZZ(tableParent, modelDataDetail);
			case EQ_Qualification_SJZZ:
				return initQualification_SJZZ(tableParent, modelDataDetail);
			case EQ_Qualification_SJ_GCJX:
				return initQualification_SJ_GCJX(tableParent, modelDataDetail);
			case EQ_Qualification_SJ_JX:
				return initQualification_SJ_JX(tableParent, modelDataDetail);
			case EQ_Qualification_SurveyReport:
				return initQualification_SurveyReport(tableParent, modelDataDetail);
			case EQ_Qualification_ZLTX:
				return initQualification_ZLTX(tableParent, modelDataDetail);
			case EQ_Qualification_ZXDJ:
				return initQualification_ZXDJ(tableParent, modelDataDetail);
			case EQ_Qualification_ZYJK:
				return initQualification_ZYJK(tableParent, modelDataDetail);
//			case EQ_Supervision:
//				return initSupervision(tableParent, modelDataDetail);
//			case EQ_SurveyDesign:
//				return initSurveyDesign(tableParent, modelDataDetail);
		}
		
		return null;
	}
	
	/**
	 * @Description: 投标人基本情况
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initBidderBasicInfo(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final BidderInfoService bidderInfoService = new BidderInfoService();
			
			private FTable<BidderBasicInfo> fTable;
			
			protected void init() {
				fTable = new FTable<BidderBasicInfo>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("投标人名称", BidderBasicInfo::getCompanyName);
						new ContentColumn("注册地址", BidderBasicInfo::getRegisterLocation);
						new ContentColumn("联系人", BidderBasicInfo::getLinkMan);
						new ContentColumn("组织结构", BidderBasicInfo::getOrgStructure);
						new ContentColumn("法人姓名", BidderBasicInfo::getLegalRepresentativeName);
						new ContentColumn("技术负责人姓名", BidderBasicInfo::getTechnicalDirectorName);
						new ContentColumn("成立时间", BidderBasicInfo::getSetupTime);
						new ContentColumn("企业资质等级", BidderBasicInfo::getCompanyQualificationLevel);
						new ContentColumn("注册资金（万元）", BidderBasicInfo::getRegisteredCapital);
						new ContentColumn("营业执照号", BidderBasicInfo::getBussinessLicense);
						new ContentColumn("开户银行", BidderBasicInfo::getDepositBank);
						new ContentColumn("银行账号", BidderBasicInfo::getBankAccount);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCompanyName() + "基本情况表"));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<BidderBasicInfo> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("投标人名称", BidderBasicInfo::getCompanyName); t.add("注册地址", BidderBasicInfo::getRegisterLocation);
						t.add("联系人", BidderBasicInfo::getLinkMan); t.add("电话", BidderBasicInfo::getLinkManPhone);
						t.add("邮政编码", BidderBasicInfo::getPostalCode); t.add("传真", BidderBasicInfo::getLinkManFax);
						t.add("网址", BidderBasicInfo::getLinkManWebsite); t.add("组织结构", BidderBasicInfo::getOrgStructure);
						t.add("法人姓名", BidderBasicInfo::getLegalRepresentativeName); t.add("技术负责人姓名", BidderBasicInfo::getTechnicalDirectorName);
						t.add("法人技术职称", BidderBasicInfo::getLegalRepresentativeTitle); t.add("技术负责人职称", BidderBasicInfo::getTechnicalDirectorTitle);
						t.add("法人电话", BidderBasicInfo::getLegalRepresentativePhone); t.add("技术负责人电话", BidderBasicInfo::getTechnicalDirectorPhone);
						t.add("成立时间", BidderBasicInfo::getSetupTime); t.add("企业资质等级", BidderBasicInfo::getCompanyQualificationLevel);
						t.add("注册资金（万元）", BidderBasicInfo::getRegisteredCapital); t.add("营业执照号", BidderBasicInfo::getBussinessLicense);
						t.add("开户银行", BidderBasicInfo::getDepositBank); t.add("银行账号", BidderBasicInfo::getBankAccount);
						t.add("高级职称人数", BidderBasicInfo::getSeniorProfessionalPostNumber); t.add("项目经理人数", BidderBasicInfo::getPurchaserNumber);
						t.add("中级职称人数", BidderBasicInfo::getMediumProfessionalPostNumber); t.add("技工人数", BidderBasicInfo::getArtisanNumber);
						t.add("初级职称人数", BidderBasicInfo::getPrimaryProfessionalPostNumber); t.add("员工总人数", BidderBasicInfo::getEmployeeNumber);
						t.addLong("企业性质", BidderBasicInfo::getEnterpriseNature);
						t.addLong("经营范围", BidderBasicInfo::getBusinessScope);
						t.addLong("股权结构", BidderBasicInfo::getEquityStructure);
						t.addBig("拟投入项目设备", BidderBasicInfo::getPreInputDevice);
						t.addBig("关联企业情况", BidderBasicInfo::getRelatedCompanyInfo);
						t.addBig("服务能力", BidderBasicInfo::getServiceAbility);
						t.addBig("备注", BidderBasicInfo::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> bidderInfoService.getBidderBasicInfoList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 营业执照
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initBusinessLicense(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final BusinessLicenseService businessLicenseService = new BusinessLicenseService();
			
			private FTable<BusinessLicense> fTable;
			
			protected void init() {
				fTable = new FTable<BusinessLicense>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("统一社会信用代码", BusinessLicense::getRegisterNumber);
						new ContentColumn("公司名称", BusinessLicense::getCompanyName);
						new ContentColumn("住所", BusinessLicense::getCompanyAddress);
						new ContentColumn("法定代表人", BusinessLicense::getLegalRepresentative);
						new ContentColumn("注册资本（万元）", BusinessLicense::getRegisterCapital);
						new ContentColumn("成立日期", BusinessLicense::getEstablishDate);
						new ContentColumn("营业期限", d -> d.getBusnissTermStartDate() + "至" + d.getBusnissTermEndtDate());
						new ContentColumn("登记机关", BusinessLicense::getRegisterInstitution);
						new ContentColumn("企业类型", BusinessLicense::getCompanyType);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCompanyName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<BusinessLicense> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("统一社会信用代码", BusinessLicense::getRegisterNumber);
						t.add("公司名称", BusinessLicense::getCompanyName);
						t.add("住所", BusinessLicense::getCompanyAddress);
						t.add("法定代表人", BusinessLicense::getLegalRepresentative);
						t.add("注册资本（万元）", BusinessLicense::getRegisterCapital);
						t.add("成立日期", BusinessLicense::getEstablishDate);
						t.add("营业期限", d -> d.getBusnissTermStartDate() + "至" + d.getBusnissTermEndtDate());
						t.add("登记机关", BusinessLicense::getRegisterInstitution);
						t.add("企业类型", BusinessLicense::getCompanyType);
						t.addBig("经营范围", BusinessLicense::getBusinessScope);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> businessLicenseService.getBusinessLicenseList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 施工业绩
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initConstruction(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final PerformanceService performanceService = new PerformanceService();
			
			private FTable<Performance> fTable;
			
			protected void init() {
				fTable = new FTable<Performance>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Performance::getSupplierName);
						new ContentColumn("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						new ContentColumn("项目名称", Performance::getProjectName);
						new ContentColumn("项目所在地", Performance::getProjectAddress);
						new ContentColumn("发包人名称", Performance::getBuyerName);
						new ContentColumn("合同金额（万元）", Performance::getSingningTotal);
						new ContentColumn("开工日期", Performance::getStartTime);
						new ContentColumn("竣工/计划竣工日期", Performance::getEndTime);
						new ContentColumn("项目经理", Performance::getProjectManager);
						new ContentColumn("技术负责人", Performance::getTechnicalDirector);
						new ContentColumn("总监理工程师", Performance::getChiefSupervisionEngineer);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getProjectName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Performance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Performance::getSupplierName); t.add("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						t.add("项目名称", Performance::getProjectName); t.add("项目所在地", Performance::getProjectAddress);
						t.add("发包人名称", Performance::getBuyerName); t.add("发包人地址", Performance::getBuyerAddress);
						t.add("发包人电话", Performance::getBuyerPhoneNumber); t.add("合同金额（万元）", Performance::getSingningTotal);
						t.add("开工日期", Performance::getStartTime); t.add("竣工/计划竣工日期", Performance::getEndTime);
						t.addLong("承担的工作", Performance::getWorkUndertaken);
						t.addLong("项目质量", Performance::getProjectQuantity);
						t.add("项目经理", Performance::getProjectManager); t.add("技术负责人", Performance::getTechnicalDirector);
						t.add("总监理工程师", Performance::getChiefSupervisionEngineer); t.add("总监理工程师电话", Performance::getCsePhoneNumber);
						t.addBig("项目描述", Performance::getProjectDesc);
						t.addLong("备注", Performance::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
				RxSwt.run(() -> performanceService.getPerformanceList(modelData.getTenderId(), modelData.getId(), statue)).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 财务报告
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initFinancialReport(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final FinanceService financeService = new FinanceService();
			
			private FTable<Finance> fTable;
			
			protected void init() {
				
				final Function<BigDecimal, String> toString = t -> Optional.ofNullable(t).map(NumberFormatUtils::format).orElse(null);
				
				fTable = new FTable<Finance>(tableParent, SWT.NONE) {
					
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Finance::getCompanyName);
						new ContentColumn("财务报告年度", Finance::getAnnual);
						new ContentColumn("资产总额（万元）", toString.compose(Finance::getTotal));
						new ContentColumn("负债总额（万元）", toString.compose(Finance::getTotalliabilities));
						new ContentColumn("资产负债率（%）", toString.compose(Finance::getLiabilitiesRate));
						new ContentColumn("主营业务收入（万元）", toString.compose(Finance::getPrimeOperatingRevenue));
						new ContentColumn("净利润（万元）", toString.compose(Finance::getRetainedProfits));
						new ContentColumn("净利率（%）", toString.compose(Finance::getProfitRatio));
					}
				};
				
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getAnnual() + "年度财务报告"));
				
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Finance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Finance::getCompanyName);
						t.add("财务报告年度", Finance::getAnnual);
						t.add("资产总额（万元）", toString.compose(Finance::getTotal));
						t.add("主营业务收入（万元）", toString.compose(Finance::getPrimeOperatingRevenue));
						t.add("负债总额（万元）", toString.compose(Finance::getTotalliabilities));
						t.add("净利润（万元）", toString.compose(Finance::getRetainedProfits));
						t.add("资产负债率（%）", toString.compose(Finance::getLiabilitiesRate));
						t.add("净利率（%）", toString.compose(Finance::getProfitRatio));
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> financeService.getFinanceList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 采购业绩
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initProcurement(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final PerformanceService performanceService = new PerformanceService();
			
			private FTable<Performance> fTable;
			
			protected void init() {
				fTable = new FTable<Performance>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Performance::getSupplierName);
						new ContentColumn("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						new ContentColumn("设备名称", Performance::getDeviceName);
						new ContentColumn("规格和型号", Performance::getDevicetypeSpecification);
						new ContentColumn("项目名称", Performance::getProjectName);
						new ContentColumn("总容量", Performance::getTotalCapacity);
						new ContentColumn("台数", Performance::getNumber);
						new ContentColumn("买方名称", Performance::getBuyerName);
						new ContentColumn("联系人", Performance::getBuyerLinkman);
						new ContentColumn("联系方式", Performance::getBuyerPhoneNumber);
						new ContentColumn("合同时间", Performance::getContractTime);
						new ContentColumn("签约合同价（万元）", Performance::getSingningTotal);
						new ContentColumn("投运/计划投运日期", Performance::getCommissioningTime);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getSupplierName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Performance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Performance::getSupplierName); t.add("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						t.add("设备名称", Performance::getDeviceName); t.add("规格和型号", Performance::getDevicetypeSpecification);
						t.add("项目名称", Performance::getProjectName); t.add("总容量", Performance::getTotalCapacity);
						t.add("台数", Performance::getNumber); t.add("买方名称", Performance::getBuyerName);
						t.add("联系人", Performance::getBuyerLinkman); t.add("联系方式", Performance::getBuyerPhoneNumber);
						t.add("合同时间", Performance::getContractTime); t.add("签约合同价（万元）", Performance::getSingningTotal);
						t.add("投运/计划投运日期", Performance::getCommissioningTime); t.addNull();
						t.addBig("备注", Performance::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
				RxSwt.run(() -> performanceService.getPerformanceList(modelData.getTenderId(), modelData.getId(), statue)).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 生产业绩
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initProduction(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final PerformanceService performanceService = new PerformanceService();
			
			private FTable<Performance> fTable;
			
			protected void init() {
				fTable = new FTable<Performance>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Performance::getSupplierName);
						new ContentColumn("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						new ContentColumn("项目名称", Performance::getProjectName);
						new ContentColumn("项目所在地", Performance::getProjectAddress);
						new ContentColumn("发包人名称", Performance::getBuyerName);
						new ContentColumn("签约价格（万元）", Performance::getSingningTotal);
						new ContentColumn("开工日期", Performance::getStartTime);
						new ContentColumn("竣工/计划竣工日期", Performance::getEndTime);
						new ContentColumn("项目经理", Performance::getProjectManager);
						new ContentColumn("技术负责人", Performance::getTechnicalDirector);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getProjectName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Performance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Performance::getSupplierName); t.add("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						t.add("项目名称", Performance::getProjectName); t.add("项目所在地", Performance::getProjectAddress);
						t.add("发包人名称", Performance::getBuyerName); t.add("发包人地址", Performance::getBuyerAddress);
						t.add("发包人电话", Performance::getBuyerPhoneNumber); t.add("签约价格（万元）", Performance::getSingningTotal);
						t.add("开工日期", Performance::getStartTime); t.add("竣工/计划竣工日期", Performance::getEndTime);
						t.addLong("承担的工作", Performance::getWorkUndertaken);
						t.addLong("项目质量", Performance::getProjectQuantity);
						t.add("项目经理", Performance::getProjectManager); t.add("技术负责人", Performance::getTechnicalDirector);
						t.addBig("项目描述", Performance::getProjectDesc);
						t.addLong("备注", Performance::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
				RxSwt.run(() -> performanceService.getPerformanceList(modelData.getTenderId(), modelData.getId(), statue)).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 项目负责人
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initProjectLeader(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final ProjectLeaderService projectLeaderService = new ProjectLeaderService();
			private final CertificateService certificateService = new CertificateService();
			private final WorkExperienceService workExperienceService = new WorkExperienceService();
			
			private FTable<ProjectLeader> fTable;
			
			protected void init() {
				fTable = new FTable<ProjectLeader>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("姓名", ProjectLeader::getLeaderName);
						new ContentColumn("身份证号", ProjectLeader::getIdnumber);
						new ContentColumn("年龄", ProjectLeader::getAge);
						new ContentColumn("职务", ProjectLeader::getJobTitle);
						new ContentColumn("职称", ProjectLeader::getProfessionalTitle);
						new ContentColumn("学历", ProjectLeader::getEducation);
						new ContentColumn("毕业学校", ProjectLeader::getGraduateSchool);
						new ContentColumn("专业", ProjectLeader::getGraduateMajor);
						new ContentColumn("工作经验（年）", ProjectLeader::getWorkExperience);
						new ContentColumn("公司名称", ProjectLeader::getCompanyName);
						new ContentColumn("本专业工作经验（年）", ProjectLeader::getCurrentMajorExperience);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCompanyName() + "基本情况表"));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<ProjectLeader> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("姓名", ProjectLeader::getLeaderName); t.add("身份证号", ProjectLeader::getIdnumber);
						t.add("年龄", ProjectLeader::getAge); t.add("职务", ProjectLeader::getJobTitle);
						t.add("职称", ProjectLeader::getProfessionalTitle); t.add("学历", ProjectLeader::getEducation);
						t.add("毕业学校", ProjectLeader::getGraduateSchool); t.add("专业", ProjectLeader::getGraduateMajor);
						t.add("工作经验（年）", ProjectLeader::getWorkExperience); t.add("公司名称", ProjectLeader::getCompanyName);
						t.add("本专业工作经验（年）", ProjectLeader::getCurrentMajorExperience); t.addNull();
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addSubpage("证书列表", p -> {
					FTable<Certificate> table = new FTable<Certificate>(p, SWT.NONE) {

						private final AttachmentFileService attachmentFileService = new AttachmentFileService();
						
						protected void init() {
							new IndexColumn("序号");
							new ContentColumn("证书编号", Certificate::getCertificateNo);
							new ContentColumn("证书名称", Certificate::getCertificateName);
							new ContentColumn("证书等级", Certificate::getCertificateLevel);
							new LinkContentColumn("查看", "查看", ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/view.png"), t -> {
								String bidFilePath = Constants.getBidFilePath(t.getTenderId(), supplierId.get());
								RxSwt.run(() -> attachmentFileService.getAttachmentFileList(t.getId())).checkWidget(this).exe(attachmentFileList -> {
									Optional<File> fileOptional = Optional.ofNullable(attachmentFileList).map(List::stream).flatMap(Stream::findFirst).map(AttachmentFile::getFileUrl).map(bidFilePath::concat).map(File::new).filter(File::exists);
									if (fileOptional.isPresent()) {
										RuntimeUtils.openFIle(fileOptional.get());
									} else {
										MessageDialog.openShortMessage("文件不存在！");
									}
								});
							});
						}
					};
					fTable.addSelectionListener(t -> RxSwt.run(() -> certificateService.getCertificateList(t.getId())).checkWidget(table).exe(table::refresh));
					return table;
				});
				modelDataDetail.addSubpage("个人经历", p -> {
					FTable<WorkExperience> table = new FTable<WorkExperience>(p, SWT.NONE) {
						protected void init() {
							new IndexColumn("序号");
							new ContentColumn("投运时间/竣工时间", WorkExperience::getTime);
							new ContentColumn("项目名称", WorkExperience::getProjectName);
							new ContentColumn("容量（MW）", WorkExperience::getCapacity);
							new ContentColumn("担任职务", WorkExperience::getJobTitle);
							new ContentColumn("承担的工作（简述）", WorkExperience::getBearWork);
							new ContentColumn("发包人及联系电话", WorkExperience::getContractInfo);
						}
					};
					fTable.addSelectionListener(t -> RxSwt.run(() -> workExperienceService.getWorkExperienceList(t.getId())).checkWidget(table).exe(table::refresh));
					return table;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> projectLeaderService.getProjectLeaderList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 企业资质
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 所建工程国家级奖励
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_GJ_GCJX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发奖单位", Qualification::getIssueAuthority);
						new ContentColumn("获奖工程名称", Qualification::getAwardProjectName);
						new ContentColumn("获奖年份", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发奖单位", Qualification::getIssueAuthority);
						t.add("获奖工程名称", Qualification::getAwardProjectName);
						t.add("获奖年份", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 国家级奖项
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_GJ_JX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发奖单位", Qualification::getIssueAuthority);
						new ContentColumn("获奖年份", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发奖单位", Qualification::getIssueAuthority);
						t.add("获奖年份", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 环境管理体系认证证书
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_HBTX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("认证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 监理资质
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_JLZZ(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 全国工业生产许可证
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_QGGYSCXK(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("认证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("认证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 施工资质
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SGZZ(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 设计资质
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SJZZ(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 安全生产许可证
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SCXK(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("认证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("认证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 所建工程省部级级奖励
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SJ_GCJX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发奖单位", Qualification::getIssueAuthority);
						new ContentColumn("获奖工程名称", Qualification::getAwardProjectName);
						new ContentColumn("获奖年份", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发奖单位", Qualification::getIssueAuthority);
						t.add("获奖工程名称", Qualification::getAwardProjectName);
						t.add("获奖年份", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 省部级奖励
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SJ_JX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发奖单位", Qualification::getIssueAuthority);
						new ContentColumn("获奖年份", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发奖单位", Qualification::getIssueAuthority);
						t.add("获奖年份", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 检验报告
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_SurveyReport(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 质量体系认证证书
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_ZLTX(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("认证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 资信等级
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_ZXDJ(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("证书等级", Qualification::getAwardLevel);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("证书等级", Qualification::getAwardLevel);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 职业健康安全管理体系
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initQualification_ZYJK(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final QualificationService qualificationService = new QualificationService();
			
			private FTable<Qualification> fTable;
			
			protected void init() {
				fTable = new FTable<Qualification>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("证书名称", Qualification::getCertificateName);
						new ContentColumn("证书编号", Qualification::getCertificateNumber);
						new ContentColumn("企业名称", Qualification::getCompanyName);
						new ContentColumn("发证单位", Qualification::getIssueAuthority);
						new ContentColumn("发证日期", Qualification::getIssueDate);
						new ContentColumn("有效期至", Qualification::getPeriodValidity);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getCertificateName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Qualification> fform = new FForm<>(p, 1);
					fform.init(t -> {
						t.add("证书编号", Qualification::getCertificateNumber);
						t.add("企业名称", Qualification::getCompanyName);
						t.add("发证单位", Qualification::getIssueAuthority);
						t.add("发证日期", Qualification::getIssueDate);
						t.add("有效期至", Qualification::getPeriodValidity);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				RxSwt.run(() -> qualificationService.getQualificationList(modelData.getTenderId(), modelData.getId())).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 监理业绩
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initSupervision(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final PerformanceService performanceService = new PerformanceService();
			
			private FTable<Performance> fTable;
			
			protected void init() {
				fTable = new FTable<Performance>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Performance::getSupplierName);
						new ContentColumn("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						new ContentColumn("项目名称", Performance::getProjectName);
						new ContentColumn("项目所在地", Performance::getProjectAddress);
						new ContentColumn("签约合同价（万元）", Performance::getSingningTotal);
						new ContentColumn("委托人名称", Performance::getBuyerName);
						new ContentColumn("委托人地址", Performance::getBuyerAddress);
						new ContentColumn("委托人电话", Performance::getBuyerPhoneNumber);
						new ContentColumn("总监理工程师", Performance::getChiefSupervisionEngineer);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getProjectName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Performance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Performance::getSupplierName); t.add("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						t.add("项目名称", Performance::getProjectName); t.add("项目所在地", Performance::getProjectAddress);
						t.add("委托人名称", Performance::getBuyerName); t.add("委托人地址", Performance::getBuyerAddress);
						t.add("签约合同价（万元）", Performance::getSingningTotal); t.add("委托人电话", Performance::getBuyerPhoneNumber);
						t.add("总监理工程师", Performance::getChiefSupervisionEngineer); t.addNull();
						t.addLong("监理服务期限", Performance::getServiceDesc);
						t.addLong("监理内容", Performance::getSupervisionCcontent);
						t.addBig("项目描述", Performance::getProjectDesc);
						t.addBig("备注", Performance::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
				RxSwt.run(() -> performanceService.getPerformanceList(modelData.getTenderId(), modelData.getId(), statue)).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
	/**
	 * @Description: 勘察设计
	 * @author chenxw
	 * @param tableParent
	 * @param modelDataDetail
	 * @return 
	 */
	private static IModelData initSurveyDesign(Composite tableParent, IModelDataDetail modelDataDetail) {
		return new IModelData() {
			
			private final PerformanceService performanceService = new PerformanceService();
			
			private FTable<Performance> fTable;
			
			protected void init() {
				fTable = new FTable<Performance>(tableParent, SWT.NONE) {
					protected void init() {
						new IndexColumn("序号");
						new ContentColumn("企业名称", Performance::getSupplierName);
						new ContentColumn("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						new ContentColumn("项目名称", Performance::getProjectName);
						new ContentColumn("项目所在地", Performance::getProjectAddress);
						new ContentColumn("发包人名称", Performance::getBuyerName);
						new ContentColumn("合同价格（万元）", Performance::getSingningTotal);
						new ContentColumn("勘察设计服务期限", Performance::getServiceDesc);
						new ContentColumn("勘察设计内容", Performance::getDesignRange);
						new ContentColumn("负责人/总工程师", Performance::getChiefSupervisionEngineer);
					}
				};
				fTable.addSelectionListener(t -> modelDataDetail.refreshTitle(t.getProjectName()));
				modelDataDetail.addBasicSubpage(p -> {
					FForm<Performance> fform = new FForm<>(p, 2);
					fform.init(t -> {
						t.add("企业名称", Performance::getSupplierName); t.add("业绩状态", PerformanceStatus.keyToValueFun.compose(Performance::getStatus));
						t.add("项目名称", Performance::getProjectName); t.add("项目所在地", Performance::getProjectAddress);
						t.add("发包人名称", Performance::getBuyerName); t.add("发包人地址", Performance::getBuyerAddress);
						t.add("发包人电话", Performance::getBuyerPhoneNumber); t.add("合同价格（万元）", Performance::getSingningTotal);
						t.add("勘察设计服务期限", Performance::getServiceDesc); t.add("勘察设计内容", Performance::getDesignRange);
						t.add("负责人/总工程师", Performance::getChiefSupervisionEngineer); t.addNull();
						t.addBig("项目描述", Performance::getProjectDesc);
						t.addBig("备注", Performance::getMemo);
					});
					fTable.addSelectionListener(fform::refresh);
					return fform;
				});
				modelDataDetail.addAttachmentSubpage(c -> fTable.addSelectionListener(t -> c.accept(supplierId.get(), t.getId())));
			}
			
			public void refresh(ModelData modelData) {
				supplierId.set(modelData.getSupplierId());
				String statue = modelData.getDataCode().substring(modelData.getDataCode().length() -2, modelData.getDataCode().length());
				RxSwt.run(() -> performanceService.getPerformanceList(modelData.getTenderId(), modelData.getId(), statue)).checkWidget(fTable).exe(l -> {
					count.set(l.size());
					fTable.refresh(l);
				});
			}
		};
	}
	
}
