package com.gx.obe.struct.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.components.core.enumeration.EvaluationStepEnum;
import com.gx.obe.struct.enums.EvaluationFactorTypeEnum;
import com.gx.obe.struct.inters.IFactorClearBidContent;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.service.SupplierEvaluationFactorService;
import com.gx.obe.web.vo.FactorClearBidContent;

public class FactorClearBidContentStepHandler implements IFactorClearBidContent {
	
	private final SupplierEvaluationFactorService supplierEvaluationFactorService = new SupplierEvaluationFactorService();
	
	private final Optional<EvaluationStep> evaluationStep;
	
	private final Optional<FactorClearBidContent> factorClearBidContent;
	
	public FactorClearBidContentStepHandler(EvaluationStep evaluationStep) {
		this.evaluationStep = Optional.ofNullable(evaluationStep);
		this.factorClearBidContent = this.evaluationStep.map(EvaluationStep::getId).map(supplierEvaluationFactorService::getFactorClearBidContent);
	}
	
	@Override
	public String getTableName() {
		Optional<EvaluationFactorTypeEnum> evaluationFactorTypeEnum = evaluationStep.map(t -> t.getEvaluationParamByKey(EvaluationStepEnum.EVALUATION_FACTOR_TYPE)).map(EvaluationFactorTypeEnum.MAP::get);
		if (!evaluationFactorTypeEnum.isPresent()) {
			return null;
		}
		switch (evaluationFactorTypeEnum.get()) {
			case COMPLIANCE:
				return "符合性指标评议表";
			case BUSINESS:
				return "商务指标评议表";
			case TECHNOLOGY:
				return "技术指标评议表";
			default:
				return null;
		}
	}
	
	@Override
	public String getFileName() {
		Optional<EvaluationFactorTypeEnum> evaluationFactorTypeEnum = evaluationStep.map(t -> t.getEvaluationParamByKey(EvaluationStepEnum.EVALUATION_FACTOR_TYPE)).map(EvaluationFactorTypeEnum.MAP::get);
		if (!evaluationFactorTypeEnum.isPresent()) {
			return null;
		}
		switch (evaluationFactorTypeEnum.get()) {
			case COMPLIANCE:
				return "评议意见-" + "符合性" + ".xlsx";
			case BUSINESS:
				return "评议意见-商务" + ".xlsx";
			case TECHNOLOGY:
				return"评议意见-技术" + ".xlsx";
			default:
				return null;
		}
	}
	
	@Override
	public List<Supplier> getSupplierList() {
		return factorClearBidContent.map(FactorClearBidContent::getSupplierList).orElseGet(ArrayList::new);
	}
	
	@Override
	public List<EvaluationFactor> getFactorList() {
		return factorClearBidContent.map(FactorClearBidContent::getFactorList).orElseGet(ArrayList::new);
	}
	
	@Override
	public List<SupplierEvaluationFactor> getSupplierEvaluationFactorList() {
		return factorClearBidContent.map(FactorClearBidContent::getSupplierEvaluationFactorList).orElseGet(ArrayList::new);
	}

	@Override
	public String getContent(SupplierEvaluationFactor supplierEvaluationFactor) {
		return supplierEvaluationFactor.getOpinion();
	}
	
}
