package com.gx.obe.struct.parse;

import java.util.List;
import java.util.stream.Stream;

import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.XmlUtils;

public class BidFileIndexTreeXmlParse {
	
	private static final String BID_FILE_INDEX_TREE = "BidFileIndexTree.xml";
	
	private String rootNodeFileName;
	private int pageNumInGenerateFile;
	
	private String bidFilePath;
	private String relChapterType;
	
	public BidFileIndexTreeXmlParse(String bidFilePath, String relChapterType) {
		this.bidFilePath = bidFilePath;
		this.relChapterType = relChapterType;
		init();
	}
	
	private void init() {
		if (bidFilePath == null) {
			return;
		}
		Document document = XmlUtils.getXmlDocument(bidFilePath + BID_FILE_INDEX_TREE);
		if (document == null) {
			return;
		}
		recursion(streamElement(document.getRootElement()));
	}
	
	private boolean recursion(Stream<Element> stream) {
		if (stream == null) return false;
		return stream.anyMatch(e -> {
			if (e.elementTextTrim("chapterType").equals(relChapterType)) {
				pageNumInGenerateFile = ObjectUtils.getIntValue(e.elementTextTrim("pageNumInGenerateFile"), 0);
//				if (e.elementTextTrim("isRootNode").equals("true")) rootNodeFileName = e.elementTextTrim("fileName");
				pageNumInGenerateFile = 1;
				rootNodeFileName = e.elementTextTrim("fileName");
				return true;
			}
			boolean recursion = recursion(streamElement(e));
			if (recursion && e.elementTextTrim("isRootNode").equals("true")) rootNodeFileName = e.elementTextTrim("fileName");
			return recursion;
		});
	}
	
	private Stream<Element> streamElement(Element element) {
		List<?> list = element.elements("fileItem");
		if (list == null) return null;
		return list.stream().map(t -> (Element) t);
	}
	
	public String getRootNodeFilePath() {
		return bidFilePath + rootNodeFileName;
	}
	
	public int getPageNumInGenerateFile() {
		return pageNumInGenerateFile;
	}
	
}
