package com.gx.obe.struct.parse;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.dom4j.Element;

import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.evaluation.struct.enumeration.StructBusinessTypeEnum;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.ParamValueDO;
import com.gx.obe.struct.beans.SelfEvalParamsDO;
import com.gx.obe.struct.value_list.CreditGrade;
import com.gx.obe.struct.value_list.Finance;
import com.gx.obe.struct.value_list.ParamValues;
import com.gx.obe.struct.value_list.Performance;
import com.gx.obe.util.element.ElementClass;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.web.service.SupplierEvaluationFactorService;

/**
 * @Description: SelfEvaluates.xml文件解析
 * @author chenxw
 */
public class SelfEvaluatesXmlParse {
	
	private final SupplierEvaluationFactorService supplierEvaluationFactorService = new SupplierEvaluationFactorService();
	
	private final String tenderId;
	/**
	 * 
	 * 
	 */
	private final Map<String, Map<String, SupplierEvaluationFactor>> mapByFactorCodeBySupplierId;
	
	/**
	 * @author chenxw
	 * @param tenderId
	 */
	public SelfEvaluatesXmlParse(String tenderId) {
		this.tenderId = tenderId;
		this.mapByFactorCodeBySupplierId = initMapByFactorCodeBySupplierId();
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 * @return
	 */
	private Map<String, Map<String, SupplierEvaluationFactor>> initMapByFactorCodeBySupplierId() {
		List<SupplierEvaluationFactor> supplierEvaluationFactorList = supplierEvaluationFactorService.getEntityListByTenderId(tenderId);
		if (CollectionUtils.isNull(supplierEvaluationFactorList)) return new HashMap<>();
		return supplierEvaluationFactorList.stream().collect(Collectors.groupingBy(SupplierEvaluationFactor::getSupplierId, Collectors.toMap(SupplierEvaluationFactor::getFactorCode, t -> t, (a, b) -> a)));
	}
	
	/**
	 * @Description: 解析
	 * @author chenxw
	 * @param supplierId
	 * @param element
	 */
	public void parse(String supplierId, Element element) {
		Map<String, SupplierEvaluationFactor> mapByFactorCode = mapByFactorCodeBySupplierId.computeIfAbsent(supplierId, $ -> new HashMap<>());
		List<?> elements = element.elements();
		elements.stream().map(t -> (Element) t).forEach(t -> {
			SupplierEvaluationFactor supplierEvaluationFactor = mapByFactorCode.computeIfAbsent(t.elementTextTrim("evalFactorId"), evalFactorCode -> {
				SupplierEvaluationFactor factor = new SupplierEvaluationFactor();
				factor.setTenderId(tenderId);
				factor.setSupplierId(supplierId);
				factor.setFactorCode(evalFactorCode);
				getSelfItemData(factor, t);
				return factor;
			});
			String value = t.elementTextTrim("selfEvaluateValue");
			supplierEvaluationFactor.setOpinion(value);
			supplierEvaluationFactor.setFactorResponse(value);
			
			Element paramValueListElement = t.element("paramValueList");
			if(null != paramValueListElement && paramValueListElement.elements().size() > 0) {
				List<Element> paramValueElementList = paramValueListElement.elements();
				if(null != paramValueElementList && paramValueElementList.size() > 0) {
					SelfEvalParamsDO selfEvalParamsDO = new SelfEvalParamsDO();
					
					for(Element paramValueElement : paramValueElementList) {
						ParamValueDO paramValueDO = new ParamValueDO();
						paramValueDO.setDataCode(paramValueElement.elementTextTrim("dataCode"));
						paramValueDO.setDataType(paramValueElement.elementTextTrim("dataType"));
						paramValueDO.setDataValue(paramValueElement.elementTextTrim("dataValue"));
						selfEvalParamsDO.addParamValue(paramValueDO);
					}
					supplierEvaluationFactor.setParamValues(selfEvalParamsDO.toJson());
				}
			}
			
		});
	}
	private void getSelfItemData(SupplierEvaluationFactor factor, Element t) {
		String relChapterType = t.elementTextTrim("relChapterType");
		Element valueList = t.element("valueList");
		if(null != valueList) {
			// 主营利润率
			if(StructBusinessTypeEnum.EQ_PROFIT_RATIO.getKey().equals(relChapterType)) {
				List<Finance> financeList= ElementClass.of(Finance.class).createList(valueList);
				String financeListJsonStr = JsonUtil.jacksonObjectToStr(financeList);
				factor.setFinance(financeListJsonStr);
//				供货业绩（物资)
			}else if(StructBusinessTypeEnum.EQ_ACHIEVEMENTS.getKey().equals(relChapterType)) {
				List<Performance> performanceList= ElementClass.of(Performance.class).createList(valueList);
				String performanceListJsonStr = JsonUtil.jacksonObjectToStr(performanceList);
				factor.setPerformance(performanceListJsonStr);
//				资信等级 EQ_Qualification_ZXDJ
			}else if(StructBusinessTypeEnum.EQ_QUALIFICATION_ZXDJ.getKey().equals(relChapterType)) {
				List<CreditGrade> creditGradeList= ElementClass.of(CreditGrade.class).createList(valueList);
				String creditGradeListJsonStr = JsonUtil.jacksonObjectToStr(creditGradeList);
				factor.setCreditGrade(creditGradeListJsonStr);
			}else {
				List<ParamValues> paramValuesList= ElementClass.of(ParamValues.class).createList(valueList);
				String paramValuesListJsonStr = JsonUtil.jacksonObjectToStr(paramValuesList);
				factor.setParamValues(paramValuesListJsonStr);
			}
		}
	}

	/**
	 * @Description: 保存
	 * @author chenxw
	 * @return
	 */
	public boolean save() {
		List<SupplierEvaluationFactor> supplierEvaluationFactorList = mapByFactorCodeBySupplierId.values().stream().map(Map::values).flatMap(Collection::stream).collect(Collectors.toList());
		if (supplierEvaluationFactorList.isEmpty()) return true;
		return supplierEvaluationFactorService.batchSaveOrUpdate(supplierEvaluationFactorList, new String[] { "opinion", "factorResponse", "paramValues" }) > 0;
	}
	
}
