package com.gx.obe.struct.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gx.obe.component.comparison.ComparisonTable;
import com.gx.obe.component.comparison.OtherMessageSHowTable;
import com.gx.obe.component.utils.CompositeUtils;
import com.gx.obe.struct.beans.StructDateInfo;
import com.gx.obe.web.entity.Result.RequestBody;
import com.gx.obe.web.entity.evaluation.EvaluationContent;
import com.gx.obe.web.utils.ObeHttpUtils;

public class EvaluationContentService {
	
	private static final String URL = "obeEvaluationContent";
	
	/**
	 * @Description: 获取评审内容列表
	 * @author chenxw
	 * @param tenderId
	 * @return
	 */
	public List<EvaluationContent> getEvaluationContentList(String tenderId) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		return ObeHttpUtils.getList(URL.concat("/getEvaluationContentList"), param, EvaluationContent.class);
	}
	
	/**
	 * @Description: 根据指标编号获取评审内容列表
	 * @author chenxw
	 * @param tenderId
	 * @param factorCode
	 * @return
	 */
	public List<EvaluationContent> getEvaluationContentListByFactorCode(String tenderId, String factorCode) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		param.put("factorCode", factorCode);
		List<EvaluationContent> list =  ObeHttpUtils.getList(URL.concat("/getEvaluationContentListByFactorCode"), param, EvaluationContent.class);
		List<EvaluationContent> list1 = new ArrayList<>();
		for(EvaluationContent evaluationContent : list) {
			if(evaluationContent.getDataCategory().equals("MD") && ( evaluationContent.getRelChapterType().equals("EQ_BidderBasicInfo") || evaluationContent.getRelChapterType().equals("EQ_FinancialReport") || evaluationContent.getRelChapterType().contains("EQ_Performance_"))) {
				list1.add(evaluationContent);
			}if(evaluationContent.getDataCategory().equals("TT")) {
				list1.add(evaluationContent);
			}
		}
		return list1;
	}
	
	/**
	 * @Description: 清空结构化信息数据
	 * @author chenxw
	 * @param tenderId
	 * @return
	 */
	public boolean deleteStructDateInfo(String tenderId) {
		Map<String, Object> param = new HashMap<>();
		param.put("tenderId", tenderId);
		return ObeHttpUtils.getBoolean(URL.concat("/deleteStructDateInfo"), param);
	}
	
	/**
	 * @Description: 保存结构化信息数据
	 * @author chenxw
	 * @param structDateInfo
	 */
	public boolean saveStructDateInfo(StructDateInfo structDateInfo) {
		return ObeHttpUtils.post(URL.concat("/saveStructDateInfo"), structDateInfo);
	}
	
	/**
	 * @Description: 清空并保存结构化信息数据
	 * @author chenxw
	 * @param tenderId
	 * @param structDateInfo
	 * @return
	 */
	public boolean deleteOrSaveStructDateInfo(String tenderId, StructDateInfo structDateInfo) {
		RequestBody<StructDateInfo> requestBody = new RequestBody<StructDateInfo>(structDateInfo);
		requestBody.setParam("tenderId", tenderId);
		return ObeHttpUtils.post(URL.concat("/deleteOrSaveStructDateInfo"), requestBody);
	}
	
}
