package com.gx.obe.struct.thread;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import org.apache.commons.lang.StringUtils;
import org.dom4j.Element;

import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.RespItemDO;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierEvaluationFactorService;

public class ParsingSupplierRespItemThread extends MyThread{

	
	
	private TenderProjectEntity tenderproject;
	private List<Supplier> supplierList;
	private SupplierEvaluationFactorService supplierEvaluationFactorService = new SupplierEvaluationFactorService();
	private Map<String, Map<String, SupplierEvaluationFactor>> mapByFactorCodeBySupplierId;
	private ProgressMessageListener messageProgress;
	
	public ParsingSupplierRespItemThread(TenderProjectEntity tenderproject, List<Supplier> supplierList, ProgressMessageListener messageProgress) {
		this.tenderproject = tenderproject;
		this.supplierList = supplierList;
		this.messageProgress = messageProgress;
	}
	
	public void run() {
		messageProgress.message("正在解析标书定位...", 0);
		this.mapByFactorCodeBySupplierId = initMapByFactorCodeBySupplierId();
		int count = 0;
		if(null != supplierList && supplierList.size() > 0) {
			for(Supplier supplier : supplierList) {
				if(parsingSupplierRespItem(supplier)) {
					count ++;
				}
			}
		}
		messageProgress.message("标书定位解析结束！", count > 0 ? 1 : -1);
		messageProgress.finish(count > 0);
	
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 * @return
	 */
	private Map<String, Map<String, SupplierEvaluationFactor>> initMapByFactorCodeBySupplierId() {
		List<SupplierEvaluationFactor> supplierEvaluationFactorList = supplierEvaluationFactorService.getEntityListByTenderId(tenderproject.getId());
		if (CollectionUtils.isNull(supplierEvaluationFactorList)) return new HashMap<>();
		return supplierEvaluationFactorList.stream().collect(Collectors.groupingBy(SupplierEvaluationFactor::getSupplierId, Collectors.toMap(SupplierEvaluationFactor::getFactorCode, t -> t, (a, b) -> a)));
	}
	
	/**
	 * @Description: 解析
	 * @author chenxw
	 * @param supplierId
	 * @param element
	 */
	public boolean parsingSupplierRespItem(Supplier supplier) {
		messageProgress.message("正在解析"+supplier.getSupplierName()+"标书定位...", 0);
		File respItemFile = new File(Constants.getBidFilePath(tenderproject.getId(), supplier.getId()) + "RespItem.xml");
		if(respItemFile.exists()) {
			Element element = XmlUtils.getXmlDocument(respItemFile.getAbsolutePath()).getRootElement();
			//TODO
			List<?> elements = element.elements();
			List<RespItemDO> respItemDOList = new ArrayList<>();
			if(null != elements && elements.size() > 0) {
				elements.stream().map(t -> (Element) t).forEach(t -> {
					RespItemDO respItemDO = new RespItemDO();
					respItemDO.setEvalFactorId(t.attributeValue("evalFactorId"));
					respItemDO.setFileId(t.attributeValue("fileId"));
					respItemDO.setFileName(t.attributeValue("fileName"));
					respItemDO.setPageNumber(t.elementTextTrim("pageNumber"));
					respItemDO.setEndPageNumber(t.elementTextTrim("endPageNumber"));
					respItemDO.setName(t.elementTextTrim("name"));
					respItemDOList.add(respItemDO);
				});
			}
			Map<String, SupplierEvaluationFactor> mapByFactorCode = mapByFactorCodeBySupplierId.get(supplier.getId());
			Map<String, List<RespItemDO>> respItemMap = respItemDOList.stream().collect(Collectors.groupingBy(RespItemDO::getEvalFactorId));
			if(null != mapByFactorCode && null != respItemMap) {
				for(String evalFactorId : respItemMap.keySet()) {
					SupplierEvaluationFactor supplierEvaluationFactor = mapByFactorCode.get(evalFactorId);
					if(null != supplierEvaluationFactor) {
						supplierEvaluationFactor.setRespItemList(JsonUtil.jsonObjectToStr(respItemMap.get(evalFactorId)));
						//页码拼接
						List<String> pageList = respItemMap.get(evalFactorId).stream().map(RespItemDO::getPage).collect(Collectors.toList());
						String pageNumber = StringUtils.join(pageList, "、");
						supplierEvaluationFactor.setParam("pageNumber", pageNumber);
					}
				}
			}
		}else {
			messageProgress.message("未找到"+supplier.getSupplierName()+"标书定位，请先下载标书或检查标书中是否提供了标书定位！", -1);
			return false;
		}
		
		if(null != mapByFactorCodeBySupplierId.get(supplier.getId()) && mapByFactorCodeBySupplierId.get(supplier.getId()).values().size() > 0) {
			
			List<SupplierEvaluationFactor> supplierEvaluationFactorList = mapByFactorCodeBySupplierId.get(supplier.getId()).values().stream().collect(Collectors.toList());
			int count = supplierEvaluationFactorService.batchSaveOrUpdate(supplierEvaluationFactorList, new String[] { "respItemList", "computerParams"});
			if(count > 0) {
				messageProgress.message(supplier.getSupplierName()+"标书定位解析成功！", 1);
				return true;
			}
		}
		messageProgress.message(supplier.getSupplierName()+"标书定位解析失败！", -1);
		return false;
	}
	

	

}