package com.gx.obe.struct.thread;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.dom4j.Element;

import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.components.core.Constants;
import com.gx.obe.evaluation.struct.enumeration.StructBusinessTypeEnum;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.struct.beans.ParamValueDO;
import com.gx.obe.struct.beans.SelfEvalParamsDO;
import com.gx.obe.struct.value_list.CreditGrade;
import com.gx.obe.struct.value_list.Finance;
import com.gx.obe.struct.value_list.ParamValues;
import com.gx.obe.struct.value_list.Performance;
import com.gx.obe.util.element.ElementClass;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierEvaluationFactorService;

public class ParsingSupplierSelfOpinionThread extends MyThread{

	
	
	private TenderProjectEntity tenderproject;
	private List<Supplier> supplierList;
	private SupplierEvaluationFactorService supplierEvaluationFactorService = new SupplierEvaluationFactorService();
	private Map<String, Map<String, SupplierEvaluationFactor>> mapByFactorCodeBySupplierId;
	private ProgressMessageListener messageProgress;
	
	public ParsingSupplierSelfOpinionThread(TenderProjectEntity tenderproject, List<Supplier> supplierList, ProgressMessageListener messageProgress) {
		this.tenderproject = tenderproject;
		this.supplierList = supplierList;
		this.messageProgress = messageProgress;
	}
	
	public void run() {
		messageProgress.message("正在解析自我评议...", 0);
		this.mapByFactorCodeBySupplierId = initMapByFactorCodeBySupplierId();
		int count = 0;
		if(null != supplierList && supplierList.size() > 0) {
			for(Supplier supplier : supplierList) {
				if(parsingSupplierSelfOpinion(supplier)) {
					count ++;
				}
			}
		}
		messageProgress.message("自我评议解析结束！", count > 0 ? 1 : -1);
		messageProgress.finish(count > 0);
	
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 * @return
	 */
	private Map<String, Map<String, SupplierEvaluationFactor>> initMapByFactorCodeBySupplierId() {
		List<SupplierEvaluationFactor> supplierEvaluationFactorList = supplierEvaluationFactorService.getEntityListByTenderId(tenderproject.getId());
		if (CollectionUtils.isNull(supplierEvaluationFactorList)) return new HashMap<>();
		return supplierEvaluationFactorList.stream().collect(Collectors.groupingBy(SupplierEvaluationFactor::getSupplierId, Collectors.toMap(SupplierEvaluationFactor::getFactorCode, t -> t, (a, b) -> a)));
	}
	
	/**
	 * @Description: 解析
	 * @author chenxw
	 * @param supplierId
	 * @param element
	 */
	public boolean parsingSupplierSelfOpinion(Supplier supplier) {
		messageProgress.message("正在解析"+supplier.getSupplierName()+"自评意见...", 0);
		File selfEvaluatesFile = new File(Constants.getBidFilePath(tenderproject.getId(), supplier.getId()) + "SelfEvaluates.xml");
		if(selfEvaluatesFile.exists()) {
			Element element = XmlUtils.getXmlDocument(selfEvaluatesFile.getAbsolutePath()).getRootElement();
			Map<String, SupplierEvaluationFactor> mapByFactorCode = mapByFactorCodeBySupplierId.computeIfAbsent(supplier.getId(), $ -> new HashMap<>());
			List<?> elements = element.elements();
			elements.stream().map(t -> (Element) t).forEach(t -> {
				SupplierEvaluationFactor supplierEvaluationFactor = mapByFactorCode.computeIfAbsent(t.elementTextTrim("evalFactorId"), evalFactorCode -> {
					SupplierEvaluationFactor factor = new SupplierEvaluationFactor();
					factor.setTenderId(tenderproject.getId());
					factor.setSupplierId(supplier.getId());
					factor.setFactorCode(evalFactorCode);
					getSelfItemData(factor, t);
					return factor;
				});
				String value = t.elementTextTrim("selfEvaluateValue");
				supplierEvaluationFactor.setOpinion(value);
				supplierEvaluationFactor.setFactorResponse(value);
				Element paramValueListElement = t.element("paramValueList");
				if(null != paramValueListElement && paramValueListElement.elements().size() > 0) {
					List<Element> paramValueElementList = paramValueListElement.elements();
					if(null != paramValueElementList && paramValueElementList.size() > 0) {
						SelfEvalParamsDO selfEvalParamsDO = new SelfEvalParamsDO();
						
						for(Element paramValueElement : paramValueElementList) {
							ParamValueDO paramValueDO = new ParamValueDO();
							paramValueDO.setDataCode(paramValueElement.elementTextTrim("dataCode"));
							paramValueDO.setDataType(paramValueElement.elementTextTrim("dataType"));
							paramValueDO.setDataValue(paramValueElement.elementTextTrim("dataValue"));
							selfEvalParamsDO.addParamValue(paramValueDO);
						}
						supplierEvaluationFactor.setParamValues(selfEvalParamsDO.toJson());
					}
				}
			});
		}else {
			messageProgress.message("未找到"+supplier.getSupplierName()+"自评意见，请先下载标书或检查标书中是否提供了自评意见！", -1);
			return false;
		}
		
		if(null != mapByFactorCodeBySupplierId.get(supplier.getId()) && mapByFactorCodeBySupplierId.get(supplier.getId()).values().size() > 0) {
			
			List<SupplierEvaluationFactor> supplierEvaluationFactorList = mapByFactorCodeBySupplierId.get(supplier.getId()).values().stream().collect(Collectors.toList());
			int count = supplierEvaluationFactorService.batchSaveOrUpdate(supplierEvaluationFactorList, new String[] { "opinion", "factorResponse", "paramValues" });
			if(count > 0) {
				messageProgress.message(supplier.getSupplierName()+"自评意见解析成功！", 1);
				return true;
			}
		}
		messageProgress.message(supplier.getSupplierName()+"自评意见解析失败！", -1);
		return false;
	}
	private void getSelfItemData(SupplierEvaluationFactor factor, Element t) {
		String relChapterType = t.elementTextTrim("relChapterType");
		Element valueList = t.element("valueList");
		if(null != valueList) {
			// 主营利润率
			if(StructBusinessTypeEnum.EQ_PROFIT_RATIO.getKey().equals(relChapterType)) {
				List<Finance> financeList= ElementClass.of(Finance.class).createList(valueList);
				String financeListJsonStr = JsonUtil.jacksonObjectToStr(financeList);
				factor.setFinance(financeListJsonStr);
//				供货业绩（物资)
			}else if(StructBusinessTypeEnum.EQ_ACHIEVEMENTS.getKey().equals(relChapterType)) {
				List<Performance> performanceList= ElementClass.of(Performance.class).createList(valueList);
				String performanceListJsonStr = JsonUtil.jacksonObjectToStr(performanceList);
				factor.setPerformance(performanceListJsonStr);
//				资信等级 EQ_Qualification_ZXDJ
			}else if(StructBusinessTypeEnum.EQ_QUALIFICATION_ZXDJ.getKey().equals(relChapterType)) {
				List<CreditGrade> creditGradeList= ElementClass.of(CreditGrade.class).createList(valueList);
				String creditGradeListJsonStr = JsonUtil.jacksonObjectToStr(creditGradeList);
				factor.setCreditGrade(creditGradeListJsonStr);
			}else {
				List<ParamValues> paramValuesList= ElementClass.of(ParamValues.class).createList(valueList);
				String paramValuesListJsonStr = JsonUtil.jacksonObjectToStr(paramValuesList);
				factor.setParamValues(paramValuesListJsonStr);
			}
		}
	}

	/**
	 * @Description: 保存
	 * @author chenxw
	 * @return
	 */
	public boolean save() {
		List<SupplierEvaluationFactor> supplierEvaluationFactorList = mapByFactorCodeBySupplierId.values().stream().map(Map::values).flatMap(Collection::stream).collect(Collectors.toList());
		if (supplierEvaluationFactorList.isEmpty()) return true;
		return supplierEvaluationFactorService.batchSaveOrUpdate(supplierEvaluationFactorList, new String[] { "opinion", "factorResponse" }) > 0;
	}

}
