package com.gx.obe.struct.tree;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.gx.obe.business.entity.SupplierEvaluationFactor;
import com.gx.obe.component.tree.ITree;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;

public class EvaluationFactorTree implements ITree<EvaluationFactorTree> {
	
	private EvaluationFactor evaluationFactor;
	private Map<String, SupplierEvaluationFactor> mapBySupplierId;
	private Map<String, Map<String, SupplierEvaluationFactor>> mapBySupplierIdByFactorCode;
	private Function<SupplierEvaluationFactor, String> mapper;
	
	public EvaluationFactorTree(EvaluationFactor evaluationFactor, Map<String, Map<String, SupplierEvaluationFactor>> mapBySupplierIdByFactorCode, Function<SupplierEvaluationFactor, String> mapper) {
		this.evaluationFactor = Objects.requireNonNull(evaluationFactor);
		this.mapBySupplierId = mapBySupplierIdByFactorCode.get(evaluationFactor.getFactorCode());
		this.mapBySupplierIdByFactorCode = mapBySupplierIdByFactorCode;
		if("对应页码".equals(evaluationFactor.getFactorName())) {
			mapper = SupplierEvaluationFactor::getPageNumber;
		}
		this.mapper = mapper;
	}
	
	@Override
	public ITree<EvaluationFactorTree> getRoot() {
		if (evaluationFactor.getParentFactor() == null) {
			return null;
		}
		return new EvaluationFactorTree(evaluationFactor.getParentFactor(), mapBySupplierIdByFactorCode, mapper);
	}
	
	@Override
	public List<EvaluationFactorTree> getITrees() {
		if (CollectionUtils.isNull(evaluationFactor.getChildFactorList())) {
			return null;
		}
		return evaluationFactor.getChildFactorList().stream().map(t -> new EvaluationFactorTree(t, mapBySupplierIdByFactorCode, mapper)).collect(Collectors.toList());
	}
	
	@Override
	public String getName() {
		return evaluationFactor.getFactorName();
	}
	
	@Override
	public String getKey() {
		return evaluationFactor.getId();
	}
	
	@Override
	public String getValue(Object key) {
		return Optional.ofNullable(mapBySupplierId).map(m -> m.get(key)).map(mapper).orElse("");
	}
	
	public EvaluationFactor getEvaluationFactor() {
		return evaluationFactor;
	}
	
	public SupplierEvaluationFactor getSupplierEvaluationFactorByKey(Object key) {
		return mapBySupplierId.get(key);
	}
	
}
